@@ -17,6 +17,7 @@
 #include "cli/mlrcli.h"
 #include "cli/quoting.h"
 #include "cli/argparse.h"
+#include "auxents/aux_entries.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -31,6 +32,8 @@
 #define DEFAULT_OQUOTING                 QUOTE_MINIMAL
 #define DEFAULT_JSON_FLATTEN_SEPARATOR   ":"
 #define DEFAULT_OOSVAR_FLATTEN_SEPARATOR ":"
+#define DEFAULT_COMMENT_STRING           "#"
+#define DEFAULT_MAX_FILE_SIZE_FOR_MMAP (4LL*1024LL*1024LL*1024LL)
 
 // ----------------------------------------------------------------
 static mapper_setup_t* mapper_lookup_table[] = {
@@ -40,6 +43,7 @@ static mapper_setup_t* mapper_lookup_table[] = {
 	&mapper_cat_setup,
 	&mapper_check_setup,
 	&mapper_count_distinct_setup,
+	&mapper_count_similar_setup,
 	&mapper_cut_setup,
 	&mapper_decimate_setup,
 	&mapper_filter_setup,
@@ -102,6 +106,7 @@ static void main_usage_help_options(FILE* o, char* argv0);
 static void main_usage_functions(FILE* o, char* argv0, char* leader);
 static void main_usage_data_format_examples(FILE* o, char* argv0);
 static void main_usage_data_format_options(FILE* o, char* argv0);
+static void main_usage_comments_in_data(FILE* o, char* argv0);
 static void main_usage_format_conversion_keystroke_saver_options(FILE* o, char* argv0);
 static void main_usage_compressed_data_options(FILE* o, char* argv0);
 static void main_usage_separator_options(FILE* o, char* argv0);
@@ -110,6 +115,7 @@ static void main_usage_double_quoting(FILE* o, char* argv0);
 static void main_usage_numerical_formatting(FILE* o, char* argv0);
 static void main_usage_other_options(FILE* o, char* argv0);
 static void main_usage_then_chaining(FILE* o, char* argv0);
+static void main_usage_auxents(FILE* o, char* argv0);
 static void main_usage_see_also(FILE* o, char* argv0);
 static void print_type_arithmetic_info(FILE* o, char* argv0);
 static void usage_all_verbs(char* argv0);
@@ -249,12 +255,32 @@ cli_opts_t* parse_command_line(int argc, char** argv, sllv_t** ppmapper_list) {
 		slls_append(popts->filenames, argv[argi], NO_FREE);
 	}
 
+	// Check for use of mmap. It's fractionally faster than stdio (due to fewer data copies
+	// -- lrecs can be pointer-backed by mmap memory) but we can't use it in all situations.
 	if (no_input) {
 		slls_free(popts->filenames);
 		popts->filenames = NULL;
 	} else if (popts->filenames->length == 0) {
 		// No filenames means read from standard input, and standard input cannot be mmapped.
 		popts->reader_opts.use_mmap_for_read = FALSE;
+	} else if (popts->reader_opts.use_mmap_for_read == TRUE) {
+		// https://github.com/johnkerl/miller/issues/160: don't use mmap for large files.
+		//
+		// If any input files don't exist, don't error out just yet ... it's possible that the user
+		// is doing some complex put-with-tee or somesuch which will create the input file by the
+		// time it's needed. In that case we of course can't know the size yet, so avoid mmap there
+		// to be safe.
+		int all_exist_and_are_small_enough = TRUE;
+		for (sllse_t* pe = popts->filenames->phead; pe != NULL; pe = pe->pnext) {
+			ssize_t file_size = get_file_size(pe->value);
+			if (file_size == (ssize_t)(-1) || file_size >= popts->reader_opts.max_file_size_for_mmap) {
+				all_exist_and_are_small_enough = FALSE;
+				break;
+			}
+		}
+		if (!all_exist_and_are_small_enough) {
+			popts->reader_opts.use_mmap_for_read = FALSE;
+		}
 	}
 
 	if (popts->do_in_place && (popts->filenames == NULL || popts->filenames->length == 0)) {
@@ -526,6 +552,10 @@ static void main_usage_long(FILE* o, char* argv0) {
 	main_usage_data_format_options(o, argv0);
 	fprintf(o, "\n");
 
+	fprintf(o, "Comments in data:\n");
+	main_usage_comments_in_data(o, argv0);
+	fprintf(o, "\n");
+
 	fprintf(o, "Format-conversion keystroke-saver options, for input, output, or both:\n");
 	main_usage_format_conversion_keystroke_saver_options(o, argv0);
 	fprintf(o, "\n");
@@ -558,6 +588,10 @@ static void main_usage_long(FILE* o, char* argv0) {
 	main_usage_then_chaining(o, argv0);
 	fprintf(o, "\n");
 
+	fprintf(o, "Auxiliary commands:\n");
+	main_usage_auxents(o, argv0);
+	fprintf(o, "\n");
+
 	main_usage_see_also(o, argv0);
 }
 
@@ -710,6 +744,7 @@ static void main_usage_data_format_options(FILE* o, char* argv0) {
 	fprintf(o, "\n");
 	fprintf(o, "  --inidx   --onidx   --nidx      Implicitly-integer-indexed fields\n");
 	fprintf(o, "                                  (Unix-toolkit style).\n");
+	fprintf(o, "  -T                              Synonymous with \"--nidx --fs tab\".\n");
 	fprintf(o, "\n");
 	fprintf(o, "  --icsv    --ocsv    --csv       Comma-separated value (or tab-separated\n");
 	fprintf(o, "                                  with --fs tab, etc.)\n");
@@ -717,6 +752,17 @@ static void main_usage_data_format_options(FILE* o, char* argv0) {
 	fprintf(o, "  --itsv    --otsv    --tsv       Keystroke-savers for \"--icsv --ifs tab\",\n");
 	fprintf(o, "                                  \"--ocsv --ofs tab\", \"--csv --fs tab\".\n");
 	fprintf(o, "\n");
+	fprintf(o, "  --icsvlite --ocsvlite --csvlite Comma-separated value (or tab-separated\n");
+	fprintf(o, "                                  with --fs tab, etc.). The 'lite' CSV does not handle\n");
+	fprintf(o, "                                  RFC-CSV double-quoting rules; is slightly faster;\n");
+	fprintf(o, "                                  and handles heterogeneity in the input stream via\n");
+	fprintf(o, "                                  empty newline followed by new header line. See also\n");
+	fprintf(o, "                                  http://johnkerl.org/miller/doc/file-formats.html#CSV/TSV/etc.\n");
+	fprintf(o, "\n");
+	fprintf(o, "  --itsvlite --otsvlite --tsvlite Keystroke-savers for \"--icsvlite --ifs tab\",\n");
+	fprintf(o, "                                  \"--ocsvlite --ofs tab\", \"--csvlite --fs tab\".\n");
+	fprintf(o, "  -t                              Synonymous with --tsvlite.\n");
+	fprintf(o, "\n");
 	fprintf(o, "  --ipprint --opprint --pprint    Pretty-printed tabular (produces no\n");
 	fprintf(o, "                                  output until all input is in).\n");
 	fprintf(o, "                      --right     Right-justifies all fields for PPRINT output.\n");
@@ -735,7 +781,6 @@ static void main_usage_data_format_options(FILE* o, char* argv0) {
 	fprintf(o, "    --json-fatal-arrays-on-input  maps. The other two options cause them to be skipped, or\n");
 	fprintf(o, "                                  to be treated as errors.  Please use the jq tool for full\n");
 	fprintf(o, "                                  JSON (pre)processing.\n");
-
 	fprintf(o, "                      --jvstack   Put one key-value pair per line for JSON\n");
 	fprintf(o, "                                  output.\n");
 	fprintf(o, "                      --jlistwrap Wrap JSON output in outermost [ ].\n");
@@ -749,10 +794,39 @@ static void main_usage_data_format_options(FILE* o, char* argv0) {
 	fprintf(o, "\n");
 	fprintf(o, "  -p is a keystroke-saver for --nidx --fs space --repifs\n");
 	fprintf(o, "\n");
+	fprintf(o, "  --mmap --no-mmap --mmap-below {n} Use mmap for files whenever possible, never, or\n");
+	fprintf(o, "                                  for files less than n bytes in size. Default is for\n");
+	fprintf(o, "                                  files less than %lld bytes in size.\n", DEFAULT_MAX_FILE_SIZE_FOR_MMAP);
+	fprintf(o, "                                  'Whenever possible' means always except for when reading\n");
+	fprintf(o, "                                  standard input which is not mmappable. If you don't know\n");
+	fprintf(o, "                                  what this means, don't worry about it -- it's a minor\n");
+	fprintf(o, "                                  performance optimization.\n");
+	fprintf(o, "\n");
 	fprintf(o, "  Examples: --csv for CSV-formatted input and output; --idkvp --opprint for\n");
 	fprintf(o, "  DKVP-formatted input and pretty-printed output.\n");
 }
 
+static void main_usage_comments_in_data(FILE* o, char* argv0) {
+	fprintf(o, "  --skip-comments                 Ignore commented lines (prefixed by \"%s\")\n",
+		DEFAULT_COMMENT_STRING);
+	fprintf(o, "                                  within the input.\n");
+	fprintf(o, "  --skip-comments-with {string}   Ignore commented lines within input, with\n");
+	fprintf(o, "                                  specified prefix.\n");
+	fprintf(o, "  --pass-comments                 Immediately print commented lines (prefixed by \"%s\")\n",
+		DEFAULT_COMMENT_STRING);
+	fprintf(o, "                                  within the input.\n");
+	fprintf(o, "  --pass-comments-with {string}   Immediately print commented lines within input, with\n");
+	fprintf(o, "                                  specified prefix.\n");
+	fprintf(o, "Notes:\n");
+	fprintf(o, "* Comments are only honored at the start of a line.\n");
+	fprintf(o, "* In the absence of any of the above four options, comments are data like\n");
+	fprintf(o, "  any other text.\n");
+	fprintf(o, "* When pass-comments is used, comment lines are written to standard output\n");
+	fprintf(o, "  immediately upon being read; they are not part of the record stream.\n");
+	fprintf(o, "  Results may be counterintuitive. A suggestion is to place comments at the\n");
+	fprintf(o, "  start of data files.\n");
+}
+
 static void main_usage_format_conversion_keystroke_saver_options(FILE* o, char* argv0) {
 	fprintf(o, "As keystroke-savers for format-conversion you may use the following:\n");
 	fprintf(o, "  --c2t --c2d --c2n --c2j --c2x --c2p --c2m\n");
@@ -905,6 +979,12 @@ static void main_usage_then_chaining(FILE* o, char* argv0) {
 	fprintf(o, "  %s stats1 -a min,mean,max -f flag,u,v -g color then sort -f color\n", argv0);
 }
 
+static void main_usage_auxents(FILE* o, char* argv0) {
+	fprintf(o, "Miller has a few otherwise-standalone executables packaged within it.\n");
+	fprintf(o, "They do not participate in any other parts of Miller.\n");
+	show_aux_entries(o);
+}
+
 static void main_usage_see_also(FILE* o, char* argv0) {
 	fprintf(o, "For more information please see http://johnkerl.org/miller/doc and/or\n");
 	fprintf(o, "http://github.com/johnkerl/miller.");
@@ -1004,6 +1084,10 @@ void cli_reader_opts_init(cli_reader_opts_t* preader_opts) {
 	preader_opts->use_mmap_for_read              = NEITHER_TRUE_NOR_FALSE;
 
 	preader_opts->prepipe                        = NULL;
+	preader_opts->comment_handling               = COMMENTS_ARE_DATA;
+	preader_opts->comment_string                 = NULL;
+
+	preader_opts->max_file_size_for_mmap         = DEFAULT_MAX_FILE_SIZE_FOR_MMAP;
 
 	// xxx temp
 	preader_opts->generator_opts.field_name     = "i";
@@ -1303,6 +1387,9 @@ static int handle_terminal_usage(char** argv, int argc, int argi) {
 	} else if (streq(argv[argi], "--usage-data-format-options")) {
 		main_usage_data_format_options(stdout, MLR_GLOBALS.bargv0);
 		return TRUE;
+	} else if (streq(argv[argi], "--usage-comments-in-data")) {
+		main_usage_comments_in_data(stdout, MLR_GLOBALS.bargv0);
+		return TRUE;
 	} else if (streq(argv[argi], "--usage-format-conversion-keystroke-saver-options")) {
 		main_usage_format_conversion_keystroke_saver_options(stdout, MLR_GLOBALS.bargv0);
 		return TRUE;
@@ -1327,6 +1414,9 @@ static int handle_terminal_usage(char** argv, int argc, int argi) {
 	} else if (streq(argv[argi], "--usage-then-chaining")) {
 		main_usage_then_chaining(stdout, MLR_GLOBALS.bargv0);
 		return TRUE;
+	} else if (streq(argv[argi], "--usage-auxents")) {
+		main_usage_auxents(stdout, MLR_GLOBALS.bargv0);
+		return TRUE;
 	} else if (streq(argv[argi], "--usage-see-also")) {
 		main_usage_see_also(stdout, MLR_GLOBALS.bargv0);
 		return TRUE;
@@ -1458,12 +1548,45 @@ int cli_handle_reader_options(char** argv, int argc, int *pargi, cli_reader_opts
 		preader_opts->use_mmap_for_read = FALSE;
 		argi += 1;
 
+	} else if (streq(argv[argi], "--mmap-below")) {
+		check_arg_count(argv, argi, argc, 2);
+		preader_opts->use_mmap_for_read = TRUE;
+		long long llmax;
+		if (sscanf(argv[argi+1], "%lld", &llmax) != 1) {
+			fprintf(stderr, "%s: could not scan \"%s\".\n",
+				MLR_GLOBALS.bargv0, argv[argi+1]);
+		}
+		preader_opts->max_file_size_for_mmap = llmax;
+		argi += 2;
+
 	} else if (streq(argv[argi], "--prepipe")) {
 		check_arg_count(argv, argi, argc, 2);
 		preader_opts->prepipe = argv[argi+1];
 		preader_opts->use_mmap_for_read = FALSE;
 		argi += 2;
 
+	} else if (streq(argv[argi], "--skip-comments")) {
+		preader_opts->comment_string = DEFAULT_COMMENT_STRING;
+		preader_opts->comment_handling = SKIP_COMMENTS;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--skip-comments-with")) {
+		check_arg_count(argv, argi, argc, 2);
+		preader_opts->comment_string = argv[argi+1];
+		preader_opts->comment_handling = SKIP_COMMENTS;
+		argi += 2;
+
+	} else if (streq(argv[argi], "--pass-comments")) {
+		preader_opts->comment_string = DEFAULT_COMMENT_STRING;
+		preader_opts->comment_handling = PASS_COMMENTS;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--pass-comments-with")) {
+		check_arg_count(argv, argi, argc, 2);
+		preader_opts->comment_string = argv[argi+1];
+		preader_opts->comment_handling = PASS_COMMENTS;
+		argi += 2;
+
 	}
 	*pargi = argi;
 	return argi != oargi;
@@ -1671,7 +1794,7 @@ int cli_handle_reader_writer_options(char** argv, int argc, int *pargi,
 		pwriter_opts->ofs = "\t";
 		argi += 1;
 
-	} else if (streq(argv[argi], "--tsvlite")) {
+	} else if (streq(argv[argi], "--tsvlite") || streq(argv[argi], "-t")) {
 		preader_opts->ifile_fmt = pwriter_opts->ofile_fmt = "csvlite";
 		preader_opts->ifs = "\t";
 		pwriter_opts->ofs = "\t";
@@ -1692,6 +1815,13 @@ int cli_handle_reader_writer_options(char** argv, int argc, int *pargi,
 		pwriter_opts->ofile_fmt = "nidx";
 		argi += 1;
 
+	} else if (streq(argv[argi], "-T")) {
+		preader_opts->ifile_fmt = "nidx";
+		pwriter_opts->ofile_fmt = "nidx";
+		preader_opts->ifs = "\t";
+		pwriter_opts->ofs = "\t";
+		argi += 1;
+
 	} else if (streq(argv[argi], "--xtab")) {
 		preader_opts->ifile_fmt = "xtab";
 		pwriter_opts->ofile_fmt = "xtab";
