@@ -9,6 +9,7 @@
 #include "containers/slls.h"
 #include "containers/sllv.h"
 #include "cli/quoting.h"
+#include "cli/comment_handling.h"
 #include "cli/json_array_ingest.h"
 #include "containers/lhmsll.h"
 #include "containers/lhmss.h"
@@ -38,7 +39,13 @@ typedef struct _cli_reader_opts_t {
 
 	// Command for popen on input, e.g. "zcat -cf <". Can be null in which case
 	// files are read directly rather than through a pipe.
-	char*  prepipe;
+	char* prepipe;
+
+	comment_handling_t comment_handling;
+	char* comment_string;
+
+	// https://github.com/johnkerl/miller/issues/160
+	ssize_t max_file_size_for_mmap;
 
 	// Fake internal-data-generator 'reader'
 	generator_opts_t generator_opts;
