@@ -22,7 +22,7 @@
 #include "lib/mlr_globals.h"
 #include "lib/mlrutil.h"
 #include "containers/lhmss.h"
-#include "containers/free_flags.h"
+#include "lib/free_flags.h"
 
 // ----------------------------------------------------------------
 // Allow compile-time override, e.g using gcc -D.
@@ -131,9 +131,8 @@ static int lhmss_find_index_for_key(lhmss_t* pmap, char* key, int* pideal_index)
 		if (++index >= pmap->array_length)
 			index = 0;
 	}
-	fprintf(stderr, "%s: internal coding error detected in file %s at line %d.\n",
-		MLR_GLOBALS.bargv0, __FILE__, __LINE__);
-	exit(1);
+	MLR_INTERNAL_CODING_ERROR();
+	return -1; // not reached
 }
 
 // ----------------------------------------------------------------
