@@ -9,6 +9,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include "cli/comment_handling.h"
 #include "lib/mlr_globals.h"
 #include "lib/mlrutil.h"
 #include "containers/slls.h"
@@ -46,6 +47,9 @@ typedef struct _lrec_reader_mmap_csvlite_state_t {
 	int   allow_repeat_ifs;
 	int   do_auto_line_term;
 	int   use_implicit_header;
+	comment_handling_t comment_handling;
+	char* comment_string;
+	int   comment_string_length;
 
 	int  expect_header_line_next;
 	header_keeper_t* pheader_keeper;
@@ -75,9 +79,19 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_single_seps_implicit_header(f
 static lrec_t* lrec_reader_mmap_csvlite_get_record_multi_seps_implicit_header(file_reader_mmap_state_t* phandle,
 	lrec_reader_mmap_csvlite_state_t* pstate, context_t* pctx, header_keeper_t* pheader_keeper, int* pend_of_stanza);
 
+static int handle_comment_line_single_irs(
+	file_reader_mmap_state_t* phandle,
+	lrec_reader_mmap_csvlite_state_t* pstate,
+	char irs);
+
+static int handle_comment_line_multi_irs(
+	file_reader_mmap_state_t* phandle,
+	lrec_reader_mmap_csvlite_state_t* pstate);
 
 // ----------------------------------------------------------------
-lrec_reader_t* lrec_reader_mmap_csvlite_alloc(char* irs, char* ifs, int allow_repeat_ifs, int use_implicit_header) {
+lrec_reader_t* lrec_reader_mmap_csvlite_alloc(char* irs, char* ifs, int allow_repeat_ifs, int use_implicit_header,
+	comment_handling_t comment_handling, char* comment_string)
+{
 	lrec_reader_t* plrec_reader = mlr_malloc_or_die(sizeof(lrec_reader_t));
 
 	lrec_reader_mmap_csvlite_state_t* pstate = mlr_malloc_or_die(sizeof(lrec_reader_mmap_csvlite_state_t));
@@ -87,8 +101,12 @@ lrec_reader_t* lrec_reader_mmap_csvlite_alloc(char* irs, char* ifs, int allow_re
 	pstate->irslen                   = strlen(irs);
 	pstate->ifslen                   = strlen(ifs);
 	pstate->allow_repeat_ifs         = allow_repeat_ifs;
-	pstate->do_auto_line_term              = FALSE;
+	pstate->do_auto_line_term        = FALSE;
 	pstate->use_implicit_header      = use_implicit_header;
+	pstate->comment_handling         = comment_handling;
+	pstate->comment_string           = comment_string;
+	pstate->comment_string_length    = comment_string == NULL ? 0 : strlen(comment_string);
+
 	pstate->expect_header_line_next  = use_implicit_header ? FALSE : TRUE;
 	pstate->pheader_keeper           = NULL;
 	pstate->pheader_keepers          = lhmslv_alloc();
@@ -243,9 +261,17 @@ static slls_t* lrec_reader_mmap_csvlite_get_header_single_seps(file_reader_mmap_
 
 	slls_t* pheader_names = slls_alloc();
 
-	while (phandle->sol < phandle->eof && *phandle->sol == irs) {
-		phandle->sol++;
-		pstate->ilno++;
+	// Skip blank/comment lines and seek to header line
+	while (TRUE) {
+		if (phandle->sol < phandle->eof && *phandle->sol == irs) {
+			phandle->sol++;
+			pstate->ilno++;
+			continue;
+		}
+		if (pstate->comment_string != NULL && handle_comment_line_single_irs(phandle, pstate, irs)) {
+			continue;
+		}
+		break;
 	}
 
 	char* p = phandle->sol;
@@ -307,13 +333,22 @@ static slls_t* lrec_reader_mmap_csvlite_get_header_multi_seps(file_reader_mmap_s
 	int   ifslen = pstate->ifslen;
 	int allow_repeat_ifs = pstate->allow_repeat_ifs;
 
-	slls_t* pheader_names = slls_alloc();
-
-	while ((phandle->eof - phandle->sol) >= irslen && streqn(phandle->sol, irs, irslen)) {
-		phandle->sol += irslen;
-		pstate->ilno++;
+	// Skip blank/comment lines and seek to header line
+	while (TRUE) {
+		if ((phandle->eof - phandle->sol) >= irslen && streqn(phandle->sol, irs, irslen)) {
+			phandle->sol += irslen;
+			pstate->ilno++;
+			continue;
+		}
+		if (pstate->comment_string != NULL && handle_comment_line_multi_irs(phandle, pstate)) {
+			continue;
+		}
+		break;
 	}
 
+	slls_t* pheader_names = slls_alloc();
+
+	// Parse the header line
 	char* p = phandle->sol;
 	if (allow_repeat_ifs) {
 		while (streqn(p, ifs, ifslen))
@@ -358,16 +393,21 @@ static slls_t* lrec_reader_mmap_csvlite_get_header_multi_seps(file_reader_mmap_s
 static lrec_t* lrec_reader_mmap_csvlite_get_record_single_seps(file_reader_mmap_state_t* phandle,
 	lrec_reader_mmap_csvlite_state_t* pstate, context_t* pctx, header_keeper_t* pheader_keeper, int* pend_of_stanza)
 {
-	if (phandle->sol >= phandle->eof)
-		return NULL;
-
 	char irs = pstate->irs[0];
 	char ifs = pstate->ifs[0];
 	int allow_repeat_ifs = pstate->allow_repeat_ifs;
 
-	lrec_t* prec = lrec_unbacked_alloc();
+	// Skip comment lines
+	if (pstate->comment_string != NULL) {
+		while (handle_comment_line_single_irs(phandle, pstate, irs))
+			;
+	}
+
+	if (phandle->sol >= phandle->eof)
+		return NULL;
 
 	char* line  = phandle->sol;
+	lrec_t* prec = lrec_unbacked_alloc();
 
 	sllse_t* pe = pheader_keeper->pkeys->phead;
 	char* p = line;
@@ -459,6 +499,11 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_single_seps(file_reader_mmap_
 static lrec_t* lrec_reader_mmap_csvlite_get_record_multi_seps(file_reader_mmap_state_t* phandle,
 	lrec_reader_mmap_csvlite_state_t* pstate, context_t* pctx, header_keeper_t* pheader_keeper, int* pend_of_stanza)
 {
+	// Skip comment lines
+	if (pstate->comment_string != NULL) {
+		while (handle_comment_line_multi_irs(phandle, pstate))
+			;
+	}
 	if (phandle->sol >= phandle->eof)
 		return NULL;
 
@@ -469,7 +514,6 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_multi_seps(file_reader_mmap_s
 	int allow_repeat_ifs = pstate->allow_repeat_ifs;
 
 	lrec_t* prec = lrec_unbacked_alloc();
-
 	char* line  = phandle->sol;
 
 	sllse_t* pe = pheader_keeper->pkeys->phead;
@@ -553,15 +597,19 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_multi_seps(file_reader_mmap_s
 static lrec_t* lrec_reader_mmap_csvlite_get_record_single_seps_implicit_header(file_reader_mmap_state_t* phandle,
 	lrec_reader_mmap_csvlite_state_t* pstate, context_t* pctx, header_keeper_t* pheader_keeper, int* pend_of_stanza)
 {
-	if (phandle->sol >= phandle->eof)
-		return NULL;
-
 	char irs = pstate->irs[0];
 	char ifs = pstate->ifs[0];
 	int allow_repeat_ifs = pstate->allow_repeat_ifs;
 
-	lrec_t* prec = lrec_unbacked_alloc();
+	// Skip comment lines
+	if (pstate->comment_string != NULL) {
+		while (handle_comment_line_single_irs(phandle, pstate, irs))
+			;
+	}
+	if (phandle->sol >= phandle->eof)
+		return NULL;
 
+	lrec_t* prec = lrec_unbacked_alloc();
 	char* line  = phandle->sol;
 
 	char* p = line;
@@ -637,6 +685,11 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_single_seps_implicit_header(f
 static lrec_t* lrec_reader_mmap_csvlite_get_record_multi_seps_implicit_header(file_reader_mmap_state_t* phandle,
 	lrec_reader_mmap_csvlite_state_t* pstate, context_t* pctx, header_keeper_t* pheader_keeper, int* pend_of_stanza)
 {
+	// Skip comment lines
+	if (pstate->comment_string != NULL) {
+		while (handle_comment_line_multi_irs(phandle, pstate))
+			;
+	}
 	if (phandle->sol >= phandle->eof)
 		return NULL;
 
@@ -647,7 +700,6 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_multi_seps_implicit_header(fi
 	int allow_repeat_ifs = pstate->allow_repeat_ifs;
 
 	lrec_t* prec = lrec_unbacked_alloc();
-
 	char* line  = phandle->sol;
 
 	char* p = line;
@@ -710,3 +762,63 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_multi_seps_implicit_header(fi
 
 	return prec;
 }
+
+// ----------------------------------------------------------------
+static int handle_comment_line_single_irs(
+	file_reader_mmap_state_t* phandle,
+	lrec_reader_mmap_csvlite_state_t* pstate,
+	char irs)
+{
+	if ((phandle->eof - phandle->sol) >= pstate->comment_string_length
+	&& streqn(phandle->sol, pstate->comment_string, pstate->comment_string_length))
+	{
+		if (pstate->comment_handling == PASS_COMMENTS)
+			for (int i = 0; i < pstate->comment_string_length; i++)
+				fputc(phandle->sol[i], stdout);
+		phandle->sol += pstate->comment_string_length;
+		while (phandle->sol < phandle->eof && *phandle->sol != irs) {
+			if (pstate->comment_handling == PASS_COMMENTS)
+				fputc(*phandle->sol, stdout);
+			phandle->sol++;
+		}
+		if (phandle->sol < phandle->eof && *phandle->sol == irs) {
+			if (pstate->comment_handling == PASS_COMMENTS)
+				fputc(*phandle->sol, stdout);
+			phandle->sol++;
+		}
+		pstate->ilno++;
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
+// ----------------------------------------------------------------
+static int handle_comment_line_multi_irs(
+	file_reader_mmap_state_t* phandle,
+	lrec_reader_mmap_csvlite_state_t* pstate)
+{
+	if ((phandle->eof - phandle->sol) >= pstate->comment_string_length
+	&& streqn(phandle->sol, pstate->comment_string, pstate->comment_string_length))
+	{
+		if (pstate->comment_handling == PASS_COMMENTS)
+			for (int i = 0; i < pstate->comment_string_length; i++)
+				fputc(phandle->sol[i], stdout);
+		phandle->sol += pstate->comment_string_length;
+		while ((phandle->eof - phandle->sol >= pstate->irslen) && !streqn(phandle->sol, pstate->irs, pstate->irslen)) {
+			if (pstate->comment_handling == PASS_COMMENTS)
+				fputc(*phandle->sol, stdout);
+			phandle->sol++;
+		}
+		if ((phandle->eof - phandle->sol >= pstate->irslen) && streqn(phandle->sol, pstate->irs, pstate->irslen)) {
+			if (pstate->comment_handling == PASS_COMMENTS)
+				for (int i = 0; i < pstate->irslen; i++)
+					fputc(phandle->sol[i], stdout);
+			phandle->sol += pstate->irslen;
+		}
+		pstate->ilno++;
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
