@@ -9,6 +9,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include "cli/comment_handling.h"
 #include "lib/mlr_globals.h"
 #include "lib/mlrutil.h"
 #include "input/file_reader_mmap.h"
@@ -23,6 +24,9 @@ typedef struct _lrec_reader_mmap_dkvp_state_t {
 	int   ipslen;
 	int   allow_repeat_ifs;
 	int   do_auto_line_term;
+	comment_handling_t comment_handling;
+	char* comment_string;
+	int   comment_string_length;
 } lrec_reader_mmap_dkvp_state_t;
 
 static void    lrec_reader_mmap_dkvp_free(lrec_reader_t* preader);
@@ -32,23 +36,51 @@ static lrec_t* lrec_reader_mmap_dkvp_process_single_irs_multi_others(void* pvsta
 static lrec_t* lrec_reader_mmap_dkvp_process_multi_irs_single_others(void* pvstate, void* pvhandle, context_t* pctx);
 static lrec_t* lrec_reader_mmap_dkvp_process_multi_irs_multi_others(void* pvstate, void* pvhandle, context_t* pctx);
 
+static lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *phandle,
+	char irs, char ifs, char ips, lrec_reader_mmap_dkvp_state_t* pstate, context_t* pctx);
+
+static lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *phandle,
+	char irs, lrec_reader_mmap_dkvp_state_t* pstate, context_t* pctx);
+
+static lrec_t* lrec_parse_mmap_dkvp_multi_irs_single_others(file_reader_mmap_state_t *phandle,
+	char ifs, char ips, lrec_reader_mmap_dkvp_state_t* pstate, context_t* pctx);
+
+static lrec_t* lrec_parse_mmap_dkvp_multi_irs_multi_others(file_reader_mmap_state_t *phandle,
+	lrec_reader_mmap_dkvp_state_t* pstate, context_t* pctx);
+
+static void skip_over_comment_lines_single_irs(
+	file_reader_mmap_state_t *phandle,
+	lrec_reader_mmap_dkvp_state_t* pstate,
+	char irs);
+
+static void skip_over_comment_lines_multi_irs(
+	file_reader_mmap_state_t *phandle,
+	lrec_reader_mmap_dkvp_state_t* pstate,
+	char* irs,
+	int irslen);
+
 // ----------------------------------------------------------------
-lrec_reader_t* lrec_reader_mmap_dkvp_alloc(char* irs, char* ifs, char* ips, int allow_repeat_ifs) {
+lrec_reader_t* lrec_reader_mmap_dkvp_alloc(char* irs, char* ifs, char* ips, int allow_repeat_ifs,
+	comment_handling_t comment_handling, char* comment_string)
+{
 	lrec_reader_t* plrec_reader = mlr_malloc_or_die(sizeof(lrec_reader_t));
 
 	lrec_reader_mmap_dkvp_state_t* pstate = mlr_malloc_or_die(sizeof(lrec_reader_mmap_dkvp_state_t));
-	pstate->irs              = irs;
-	pstate->ifs              = ifs;
-	pstate->ips              = ips;
-	pstate->irslen           = strlen(irs);
-	pstate->ifslen           = strlen(ifs);
-	pstate->ipslen           = strlen(ips);
-	pstate->allow_repeat_ifs = allow_repeat_ifs;
-	pstate->do_auto_line_term      = FALSE;
+	pstate->irs                   = irs;
+	pstate->ifs                   = ifs;
+	pstate->ips                   = ips;
+	pstate->irslen                = strlen(irs);
+	pstate->ifslen                = strlen(ifs);
+	pstate->ipslen                = strlen(ips);
+	pstate->allow_repeat_ifs      = allow_repeat_ifs;
+	pstate->do_auto_line_term     = FALSE;
+	pstate->comment_handling      = comment_handling;
+	pstate->comment_string        = comment_string;
+	pstate->comment_string_length = comment_string == NULL ? 0 : strlen(comment_string);
 
-	plrec_reader->pvstate       = (void*)pstate;
-	plrec_reader->popen_func    = file_reader_mmap_vopen;
-	plrec_reader->pclose_func   = file_reader_mmap_vclose;
+	plrec_reader->pvstate     = (void*)pstate;
+	plrec_reader->popen_func  = file_reader_mmap_vopen;
+	plrec_reader->pclose_func = file_reader_mmap_vclose;
 	if (streq(irs, "auto")) {
 		// Auto means either lines end in "\n" or "\r\n" (LF or CRLF).  In
 		// either case the final character is "\n". Then for autodetect we
@@ -92,7 +124,7 @@ static lrec_t* lrec_reader_mmap_dkvp_process_single_irs_single_others(void* pvst
 		return NULL;
 	else
 		return lrec_parse_mmap_dkvp_single_irs_single_others(phandle, pstate->irs[0], pstate->ifs[0], pstate->ips[0],
-			pstate->allow_repeat_ifs, pstate->do_auto_line_term, pctx);
+			pstate, pctx);
 }
 
 static lrec_t* lrec_reader_mmap_dkvp_process_single_irs_multi_others(void* pvstate, void* pvhandle, context_t* pctx) {
@@ -101,8 +133,7 @@ static lrec_t* lrec_reader_mmap_dkvp_process_single_irs_multi_others(void* pvsta
 	if (phandle->sol >= phandle->eof)
 		return NULL;
 	else
-		return lrec_parse_mmap_dkvp_single_irs_multi_others(phandle, pstate->irs[0], pstate->ifs, pstate->ips,
-			pstate->ifslen, pstate->ipslen, pstate->allow_repeat_ifs, pstate->do_auto_line_term, pctx);
+		return lrec_parse_mmap_dkvp_single_irs_multi_others(phandle, pstate->irs[0], pstate, pctx);
 }
 
 static lrec_t* lrec_reader_mmap_dkvp_process_multi_irs_single_others(void* pvstate, void* pvhandle, context_t* pctx) {
@@ -111,8 +142,8 @@ static lrec_t* lrec_reader_mmap_dkvp_process_multi_irs_single_others(void* pvsta
 	if (phandle->sol >= phandle->eof)
 		return NULL;
 	else
-		return lrec_parse_mmap_dkvp_multi_irs_single_others(phandle, pstate->irs, pstate->ifs[0], pstate->ips[0],
-			pstate->irslen, pstate->allow_repeat_ifs, pctx);
+		return lrec_parse_mmap_dkvp_multi_irs_single_others(phandle, pstate->ifs[0], pstate->ips[0],
+			pstate, pctx);
 }
 
 static lrec_t* lrec_reader_mmap_dkvp_process_multi_irs_multi_others(void* pvstate, void* pvhandle, context_t* pctx) {
@@ -121,21 +152,25 @@ static lrec_t* lrec_reader_mmap_dkvp_process_multi_irs_multi_others(void* pvstat
 	if (phandle->sol >= phandle->eof)
 		return NULL;
 	else
-		return lrec_parse_mmap_dkvp_multi_irs_multi_others(phandle, pstate->irs, pstate->ifs, pstate->ips,
-			pstate->irslen, pstate->ifslen, pstate->ipslen, pstate->allow_repeat_ifs, pctx);
+		return lrec_parse_mmap_dkvp_multi_irs_multi_others(phandle, pstate, pctx);
 }
 
 // ----------------------------------------------------------------
-lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *phandle,
-	char irs, char ifs, char ips, int allow_repeat_ifs, int do_auto_line_term, context_t* pctx)
+static lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *phandle,
+	char irs, char ifs, char ips, lrec_reader_mmap_dkvp_state_t* pstate, context_t* pctx)
 {
-	lrec_t* prec = lrec_unbacked_alloc();
+	if (pstate->comment_string != NULL)
+		skip_over_comment_lines_single_irs(phandle, pstate, irs);
+
+	if (phandle->sol >= phandle->eof)
+		return NULL;
 
 	char* line  = phandle->sol;
+	lrec_t* prec = lrec_unbacked_alloc();
 
 	int idx = 0;
 	char* p = line;
-	if (allow_repeat_ifs) {
+	if (pstate->allow_repeat_ifs) {
 		while (*p == ifs)
 			p++;
 	}
@@ -149,7 +184,7 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *
 		if (*p == irs) {
 			*p = 0;
 
-			if (do_auto_line_term) {
+			if (pstate->do_auto_line_term) {
 				if (p > line && p[-1] == '\r') {
 					p[-1] = 0;
 					context_set_autodetected_crlf(pctx);
@@ -178,7 +213,7 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *
 			}
 
 			p++;
-			if (allow_repeat_ifs) {
+			if (pstate->allow_repeat_ifs) {
 				while (*p == ifs)
 					p++;
 			}
@@ -197,7 +232,7 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *
 		phandle->sol = p+1;
 	idx++;
 
-	if (allow_repeat_ifs && *key == 0 && *value == 0)
+	if (pstate->allow_repeat_ifs && *key == 0 && *value == 0)
 		return prec;
 
 	// There are two ways out of that loop: saw IRS, or saw end of file.
@@ -244,16 +279,21 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *
 	return prec;
 }
 
-lrec_t* lrec_parse_mmap_dkvp_multi_irs_single_others(file_reader_mmap_state_t *phandle,
-	char* irs, char ifs, char ips, int irslen, int allow_repeat_ifs, context_t* pctx)
+static lrec_t* lrec_parse_mmap_dkvp_multi_irs_single_others(file_reader_mmap_state_t *phandle,
+	char ifs, char ips, lrec_reader_mmap_dkvp_state_t* pstate, context_t* pctx)
 {
-	lrec_t* prec = lrec_unbacked_alloc();
+	if (pstate->comment_string != NULL)
+		skip_over_comment_lines_multi_irs(phandle, pstate, pstate->irs, pstate->irslen);
+
+	if (phandle->sol >= phandle->eof)
+		return NULL;
 
 	char* line  = phandle->sol;
+	lrec_t* prec = lrec_unbacked_alloc();
 
 	int idx = 0;
 	char* p = line;
-	if (allow_repeat_ifs) {
+	if (pstate->allow_repeat_ifs) {
 		while (*p == ifs)
 			p++;
 	}
@@ -264,9 +304,9 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_single_others(file_reader_mmap_state_t *p
 	int saw_rs = FALSE;
 
 	for ( ; p < phandle->eof && *p; ) {
-		if (streqn(p, irs, irslen)) {
+		if (streqn(p, pstate->irs, pstate->irslen)) {
 			*p = 0;
-			phandle->sol = p + irslen;
+			phandle->sol = p + pstate->irslen;
 			saw_rs = TRUE;
 			break;
 		} else if (*p == ifs) {
@@ -286,7 +326,7 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_single_others(file_reader_mmap_state_t *p
 			}
 
 			p++;
-			if (allow_repeat_ifs) {
+			if (pstate->allow_repeat_ifs) {
 				while (*p == ifs)
 					p++;
 			}
@@ -305,7 +345,7 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_single_others(file_reader_mmap_state_t *p
 		phandle->sol = p+1;
 	idx++;
 
-	if (allow_repeat_ifs && *key == 0 && *value == 0)
+	if (pstate->allow_repeat_ifs && *key == 0 && *value == 0)
 		return prec;
 
 	// There are two ways out of that loop: saw IRS, or saw end of file.
@@ -352,18 +392,23 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_single_others(file_reader_mmap_state_t *p
 	return prec;
 }
 
-lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *phandle, char irs, char* ifs, char* ips,
-	int ifslen, int ipslen, int allow_repeat_ifs, int do_auto_line_term, context_t* pctx)
+static lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *phandle, char irs,
+	lrec_reader_mmap_dkvp_state_t* pstate, context_t* pctx)
 {
-	lrec_t* prec = lrec_unbacked_alloc();
+	if (pstate->comment_string != NULL)
+		skip_over_comment_lines_single_irs(phandle, pstate, irs);
+
+	if (phandle->sol >= phandle->eof)
+		return NULL;
 
 	char* line  = phandle->sol;
+	lrec_t* prec = lrec_unbacked_alloc();
 
 	int idx = 0;
 	char* p = line;
-	if (allow_repeat_ifs) {
-		while (streqn(p, ifs, ifslen))
-			p += ifslen;
+	if (pstate->allow_repeat_ifs) {
+		while (streqn(p, pstate->ifs, pstate->ifslen))
+			p += pstate->ifslen;
 	}
 	char* key   = p;
 	char* value = p;
@@ -375,7 +420,7 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *p
 		if (*p == irs) {
 			*p = 0;
 
-			if (do_auto_line_term) {
+			if (pstate->do_auto_line_term) {
 				if (p > line && p[-1] == '\r') {
 					p[-1] = 0;
 					context_set_autodetected_crlf(pctx);
@@ -387,7 +432,7 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *p
 			phandle->sol = p+1;
 			saw_rs = TRUE;
 			break;
-		} else if (streqn(p, ifs, ifslen)) {
+		} else if (streqn(p, pstate->ifs, pstate->ifslen)) {
 			saw_ps = FALSE;
 			*p = 0;
 
@@ -403,16 +448,16 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *p
 				lrec_put(prec, key, value, NO_FREE);
 			}
 
-			p += ifslen;
-			if (allow_repeat_ifs) {
-				while (streqn(p, ifs, ifslen))
-					p += ifslen;
+			p += pstate->ifslen;
+			if (pstate->allow_repeat_ifs) {
+				while (streqn(p, pstate->ifs, pstate->ifslen))
+					p += pstate->ifslen;
 			}
 			key = p;
 			value = p;
-		} else if (streqn(p, ips, ipslen) && !saw_ps) {
+		} else if (streqn(p, pstate->ips, pstate->ipslen) && !saw_ps) {
 			*p = 0;
-			p += ipslen;
+			p += pstate->ipslen;
 			value = p;
 			saw_ps = TRUE;
 		} else {
@@ -424,7 +469,7 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *p
 		phandle->sol = p+1;
 	idx++;
 
-	if (allow_repeat_ifs && *key == 0 && *value == 0)
+	if (pstate->allow_repeat_ifs && *key == 0 && *value == 0)
 		return prec;
 
 	// There are two ways out of that loop: saw IRS, or saw end of file.
@@ -471,18 +516,23 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *p
 	return prec;
 }
 
-lrec_t* lrec_parse_mmap_dkvp_multi_irs_multi_others(file_reader_mmap_state_t *phandle,
-	char* irs, char* ifs, char* ips, int irslen, int ifslen, int ipslen, int allow_repeat_ifs, context_t* pctx)
+static lrec_t* lrec_parse_mmap_dkvp_multi_irs_multi_others(file_reader_mmap_state_t *phandle,
+	lrec_reader_mmap_dkvp_state_t* pstate, context_t* pctx)
 {
-	lrec_t* prec = lrec_unbacked_alloc();
+	if (pstate->comment_string != NULL)
+		skip_over_comment_lines_multi_irs(phandle, pstate, pstate->irs, pstate->irslen);
+
+	if (phandle->sol >= phandle->eof)
+		return NULL;
 
 	char* line  = phandle->sol;
+	lrec_t* prec = lrec_unbacked_alloc();
 
 	int idx = 0;
 	char* p = line;
-	if (allow_repeat_ifs) {
-		while (streqn(p, ifs, ifslen))
-			p += ifslen;
+	if (pstate->allow_repeat_ifs) {
+		while (streqn(p, pstate->ifs, pstate->ifslen))
+			p += pstate->ifslen;
 	}
 	char* key   = p;
 	char* value = p;
@@ -491,12 +541,12 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_multi_others(file_reader_mmap_state_t *ph
 	int saw_rs = FALSE;
 
 	for ( ; p < phandle->eof && *p; ) {
-		if (streqn(p, irs, irslen)) {
+		if (streqn(p, pstate->irs, pstate->irslen)) {
 			*p = 0;
-			phandle->sol = p + irslen;
+			phandle->sol = p + pstate->irslen;
 			saw_rs = TRUE;
 			break;
-		} else if (streqn(p, ifs, ifslen)) {
+		} else if (streqn(p, pstate->ifs, pstate->ifslen)) {
 			saw_ps = FALSE;
 			*p = 0;
 
@@ -512,16 +562,16 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_multi_others(file_reader_mmap_state_t *ph
 				lrec_put(prec, key, value, NO_FREE);
 			}
 
-			p += ifslen;
-			if (allow_repeat_ifs) {
-				while (streqn(p, ifs, ifslen))
-					p += ifslen;
+			p += pstate->ifslen;
+			if (pstate->allow_repeat_ifs) {
+				while (streqn(p, pstate->ifs, pstate->ifslen))
+					p += pstate->ifslen;
 			}
 			key = p;
 			value = p;
-		} else if (streqn(p, ips, ipslen) && !saw_ps) {
+		} else if (streqn(p, pstate->ips, pstate->ipslen) && !saw_ps) {
 			*p = 0;
-			p += ipslen;
+			p += pstate->ipslen;
 			value = p;
 			saw_ps = TRUE;
 		} else {
@@ -532,7 +582,7 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_multi_others(file_reader_mmap_state_t *ph
 		phandle->sol = p+1;
 	idx++;
 
-	if (allow_repeat_ifs && *key == 0 && *value == 0)
+	if (pstate->allow_repeat_ifs && *key == 0 && *value == 0)
 		return prec;
 
 	// There are two ways out of that loop: saw IRS, or saw end of file.
@@ -578,3 +628,56 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_multi_others(file_reader_mmap_state_t *ph
 
 	return prec;
 }
+
+// ----------------------------------------------------------------
+static void skip_over_comment_lines_single_irs(
+	file_reader_mmap_state_t *phandle,
+	lrec_reader_mmap_dkvp_state_t* pstate,
+	char irs)
+{
+	while ((phandle->eof - phandle->sol) >= pstate->comment_string_length
+	&& streqn(phandle->sol, pstate->comment_string, pstate->comment_string_length))
+	{
+		if (pstate->comment_handling == PASS_COMMENTS)
+			for (int i = 0; i < pstate->comment_string_length; i++)
+				fputc(phandle->sol[i], stdout);
+		phandle->sol += pstate->comment_string_length;
+		while (phandle->sol < phandle->eof && *phandle->sol != irs) {
+			if (pstate->comment_handling == PASS_COMMENTS)
+				fputc(*phandle->sol, stdout);
+			phandle->sol++;
+		}
+		if (phandle->sol < phandle->eof && *phandle->sol == irs) {
+			if (pstate->comment_handling == PASS_COMMENTS)
+				fputc(*phandle->sol, stdout);
+			phandle->sol++;
+		}
+	}
+}
+
+static void skip_over_comment_lines_multi_irs(
+	file_reader_mmap_state_t *phandle,
+	lrec_reader_mmap_dkvp_state_t* pstate,
+	char* irs,
+	int irslen)
+{
+	while ((phandle->eof - phandle->sol) >= pstate->comment_string_length
+	&& streqn(phandle->sol, pstate->comment_string, pstate->comment_string_length))
+	{
+		if (pstate->comment_handling == PASS_COMMENTS)
+			for (int i = 0; i < pstate->comment_string_length; i++)
+				fputc(phandle->sol[i], stdout);
+		phandle->sol += pstate->comment_string_length;
+		while ((phandle->eof - phandle->sol) >= irslen && !streqn(phandle->sol, irs, irslen)) {
+			if (pstate->comment_handling == PASS_COMMENTS)
+				fputc(*phandle->sol, stdout);
+			phandle->sol++;
+		}
+		if ((phandle->eof - phandle->sol) >= irslen && streqn(phandle->sol, irs, irslen)) {
+			if (pstate->comment_handling == PASS_COMMENTS)
+				for (int i = 0; i < irslen; i++)
+					fputc(phandle->sol[i], stdout);
+			phandle->sol += irslen;
+		}
+	}
+}
