@@ -9,6 +9,8 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include "cli/comment_handling.h"
+#include "lib/mlr_globals.h"
 #include "lib/mlrutil.h"
 #include "input/file_reader_mmap.h"
 #include "input/lrec_readers.h"
@@ -29,8 +31,32 @@ static lrec_t* lrec_reader_mmap_xtab_process_single_ifs_multi_ips(void* pvstate,
 static lrec_t* lrec_reader_mmap_xtab_process_multi_ifs_single_ips(void* pvstate, void* pvhandle, context_t* pctx);
 static lrec_t* lrec_reader_mmap_xtab_process_multi_ifs_multi_ips(void* pvstate, void* pvhandle, context_t* pctx);
 
+static lrec_t* lrec_parse_mmap_xtab_single_ifs_single_ips(file_reader_mmap_state_t* phandle, char ifs, char ips,
+	lrec_reader_mmap_xtab_state_t* pstate, context_t* pctx);
+
+static lrec_t* lrec_parse_mmap_xtab_single_ifs_multi_ips(file_reader_mmap_state_t* phandle, char ifs,
+	lrec_reader_mmap_xtab_state_t* pstate, context_t* pctx);
+
+static lrec_t* lrec_parse_mmap_xtab_multi_ifs_single_ips(file_reader_mmap_state_t* phandle, char ips,
+	lrec_reader_mmap_xtab_state_t* pstate);
+
+static lrec_t* lrec_parse_mmap_xtab_multi_ifs_multi_ips(file_reader_mmap_state_t* phandle,
+	lrec_reader_mmap_xtab_state_t* pstate);
+
 // ----------------------------------------------------------------
-lrec_reader_t* lrec_reader_mmap_xtab_alloc(char* ifs, char* ips, int allow_repeat_ips) {
+lrec_reader_t* lrec_reader_mmap_xtab_alloc(char* ifs, char* ips, int allow_repeat_ips,
+	comment_handling_t comment_handling, char* comment_string)
+{
+	// lrec_reader_alloc should have shunted away from us in this case.
+	// (Interleaving blank-line handling, line-term autodetect, and comment-handling all in
+	// the byte-at-a-time logic turned out to be a mess in this file. In the stdio implementation,
+	// by constrast, it falls out rather easily.)
+	if (comment_string != NULL) {
+		fprintf(stderr, "%s: internal coding error detected in file %s at line %d.\n",
+			MLR_GLOBALS.bargv0, __FILE__, __LINE__);
+		exit(1);
+	}
+
 	lrec_reader_t* plrec_reader = mlr_malloc_or_die(sizeof(lrec_reader_t));
 
 	lrec_reader_mmap_xtab_state_t* pstate = mlr_malloc_or_die(sizeof(lrec_reader_mmap_xtab_state_t));
@@ -89,7 +115,7 @@ static lrec_t* lrec_reader_mmap_xtab_process_single_ifs_single_ips(void* pvstate
 		return NULL;
 	else
 		return lrec_parse_mmap_xtab_single_ifs_single_ips(phandle, pstate->ifs[0], pstate->ips[0],
-			pstate->allow_repeat_ips, pstate->do_auto_line_term, pctx);
+			pstate, pctx);
 }
 
 static lrec_t* lrec_reader_mmap_xtab_process_single_ifs_multi_ips(void* pvstate, void* pvhandle, context_t* pctx) {
@@ -98,8 +124,7 @@ static lrec_t* lrec_reader_mmap_xtab_process_single_ifs_multi_ips(void* pvstate,
 	if (phandle->sol >= phandle->eof)
 		return NULL;
 	else
-		return lrec_parse_mmap_xtab_single_ifs_multi_ips(phandle, pstate->ifs[0], pstate->ips, pstate->ipslen,
-			pstate->allow_repeat_ips, pstate->do_auto_line_term, pctx);
+		return lrec_parse_mmap_xtab_single_ifs_multi_ips(phandle, pstate->ifs[0], pstate, pctx);
 }
 
 static lrec_t* lrec_reader_mmap_xtab_process_multi_ifs_single_ips(void* pvstate, void* pvhandle, context_t* pctx) {
@@ -108,8 +133,7 @@ static lrec_t* lrec_reader_mmap_xtab_process_multi_ifs_single_ips(void* pvstate,
 	if (phandle->sol >= phandle->eof)
 		return NULL;
 	else
-		return lrec_parse_mmap_xtab_multi_ifs_single_ips(phandle, pstate->ifs, pstate->ips[0], pstate->ifslen,
-			pstate->allow_repeat_ips);
+		return lrec_parse_mmap_xtab_multi_ifs_single_ips(phandle, pstate->ips[0], pstate);
 }
 
 static lrec_t* lrec_reader_mmap_xtab_process_multi_ifs_multi_ips(void* pvstate, void* pvhandle, context_t* pctx) {
@@ -118,15 +142,14 @@ static lrec_t* lrec_reader_mmap_xtab_process_multi_ifs_multi_ips(void* pvstate,
 	if (phandle->sol >= phandle->eof)
 		return NULL;
 	else
-		return lrec_parse_mmap_xtab_multi_ifs_multi_ips(phandle, pstate->ifs, pstate->ips, pstate->ifslen,
-			pstate->ipslen, pstate->allow_repeat_ips);
+		return lrec_parse_mmap_xtab_multi_ifs_multi_ips(phandle, pstate);
 }
 
 // ----------------------------------------------------------------
-lrec_t* lrec_parse_mmap_xtab_single_ifs_single_ips(file_reader_mmap_state_t* phandle, char ifs, char ips,
-	int allow_repeat_ips, int do_auto_line_term, context_t* pctx)
+static lrec_t* lrec_parse_mmap_xtab_single_ifs_single_ips(file_reader_mmap_state_t* phandle, char ifs, char ips,
+	lrec_reader_mmap_xtab_state_t* pstate, context_t* pctx)
 {
-	if (do_auto_line_term) {
+	if (pstate->do_auto_line_term) {
 		// Skip over otherwise empty LF-only or CRLF-only lines.
 		while (phandle->sol < phandle->eof) {
 			if (*phandle->sol == '\n') {
@@ -145,9 +168,10 @@ lrec_t* lrec_parse_mmap_xtab_single_ifs_single_ips(file_reader_mmap_state_t* pha
 			}
 		}
 	} else {
-		// Skip over otherwise empty IRS-only lines.
-		while (phandle->sol < phandle->eof && *phandle->sol == ifs)
+		// Skip over otherwise empty IFS-only lines
+		while (phandle->sol < phandle->eof && *phandle->sol == ifs) {
 			phandle->sol++;
+		}
 	}
 
 	if (phandle->sol >= phandle->eof)
@@ -157,6 +181,9 @@ lrec_t* lrec_parse_mmap_xtab_single_ifs_single_ips(file_reader_mmap_state_t* pha
 
 	// Loop over fields, one per line
 	while (TRUE) {
+		if (phandle->sol >= phandle->eof)
+			break;
+
 		char* line  = phandle->sol;
 		char* key   = line;
 		char* value = "";
@@ -168,7 +195,7 @@ lrec_t* lrec_parse_mmap_xtab_single_ifs_single_ips(file_reader_mmap_state_t* pha
 			if (*p == ifs) {
 				*p = 0;
 
-				if (do_auto_line_term) {
+				if (pstate->do_auto_line_term) {
 					if (p > line && p[-1] == '\r') {
 						p[-1] = 0;
 						context_set_autodetected_crlf(pctx);
@@ -185,7 +212,7 @@ lrec_t* lrec_parse_mmap_xtab_single_ifs_single_ips(file_reader_mmap_state_t* pha
 				*p = 0;
 
 				p++;
-				if (allow_repeat_ips) {
+				if (pstate->allow_repeat_ips) {
 					while (*p == ips)
 						p++;
 				}
@@ -213,7 +240,7 @@ lrec_t* lrec_parse_mmap_xtab_single_ifs_single_ips(file_reader_mmap_state_t* pha
 		if (phandle->sol >= phandle->eof)
 			break;
 
-		if (do_auto_line_term) {
+		if (pstate->do_auto_line_term) {
 			char* p = phandle->sol;
 			char* q = phandle->sol + 1;
 			if (*p == '\n')
@@ -233,10 +260,10 @@ lrec_t* lrec_parse_mmap_xtab_single_ifs_single_ips(file_reader_mmap_state_t* pha
 	}
 }
 
-lrec_t* lrec_parse_mmap_xtab_single_ifs_multi_ips(file_reader_mmap_state_t* phandle, char ifs, char* ips, int ipslen,
-	int allow_repeat_ips, int do_auto_line_term, context_t* pctx)
+static lrec_t* lrec_parse_mmap_xtab_single_ifs_multi_ips(file_reader_mmap_state_t* phandle, char ifs,
+	lrec_reader_mmap_xtab_state_t* pstate, context_t* pctx)
 {
-	if (do_auto_line_term) {
+	if (pstate->do_auto_line_term) {
 		// Skip over otherwise empty LF-only or CRLF-only lines.
 		while (phandle->sol < phandle->eof) {
 			if (*phandle->sol == '\n') {
@@ -255,7 +282,7 @@ lrec_t* lrec_parse_mmap_xtab_single_ifs_multi_ips(file_reader_mmap_state_t* phan
 			}
 		}
 	} else {
-		// Skip over otherwise empty IRS-only lines.
+		// Skip over otherwise empty IFS-only lines.
 		while (phandle->sol < phandle->eof && *phandle->sol == ifs)
 			phandle->sol++;
 	}
@@ -263,10 +290,16 @@ lrec_t* lrec_parse_mmap_xtab_single_ifs_multi_ips(file_reader_mmap_state_t* phan
 	if (phandle->sol >= phandle->eof)
 		return NULL;
 
+	char* ips = pstate->ips;
+	int ipslen = pstate->ipslen;
+
 	lrec_t* prec = lrec_unbacked_alloc();
 
 	// Loop over fields, one per line
 	while (TRUE) {
+		if (phandle->sol >= phandle->eof)
+			break;
+
 		char* line  = phandle->sol;
 		char* key   = line;
 		char* value = "";
@@ -278,7 +311,7 @@ lrec_t* lrec_parse_mmap_xtab_single_ifs_multi_ips(file_reader_mmap_state_t* phan
 			if (*p == ifs) {
 				*p = 0;
 
-				if (do_auto_line_term) {
+				if (pstate->do_auto_line_term) {
 					if (p > line && p[-1] == '\r') {
 						p[-1] = 0;
 						context_set_autodetected_crlf(pctx);
@@ -295,7 +328,7 @@ lrec_t* lrec_parse_mmap_xtab_single_ifs_multi_ips(file_reader_mmap_state_t* phan
 				*p = 0;
 
 				p += ipslen;
-				if (allow_repeat_ips) {
+				if (pstate->allow_repeat_ips) {
 					while (streqn(p, ips, ipslen))
 						p += ipslen;
 				}
@@ -331,11 +364,16 @@ lrec_t* lrec_parse_mmap_xtab_single_ifs_multi_ips(file_reader_mmap_state_t* phan
 	}
 }
 
-lrec_t* lrec_parse_mmap_xtab_multi_ifs_single_ips(file_reader_mmap_state_t* phandle, char* ifs, char ips, int ifslen,
-	int allow_repeat_ips)
+static lrec_t* lrec_parse_mmap_xtab_multi_ifs_single_ips(file_reader_mmap_state_t* phandle, char ips,
+	lrec_reader_mmap_xtab_state_t* pstate)
 {
-	while (phandle->sol < phandle->eof && streqn(phandle->sol, ifs, ifslen))
+	char* ifs = pstate->ifs;
+	int ifslen = pstate->ifslen;
+
+	// Skip blank lines
+	while (phandle->eof - phandle->sol >= ifslen && streqn(phandle->sol, ifs, ifslen)) {
 		phandle->sol += ifslen;
+	}
 
 	if (phandle->sol >= phandle->eof)
 		return NULL;
@@ -344,6 +382,9 @@ lrec_t* lrec_parse_mmap_xtab_multi_ifs_single_ips(file_reader_mmap_state_t* phan
 
 	// Loop over fields, one per line
 	while (TRUE) {
+		if (phandle->sol >= phandle->eof)
+			break;
+
 		char* line  = phandle->sol;
 		char* key   = line;
 		char* value = "";
@@ -362,7 +403,7 @@ lrec_t* lrec_parse_mmap_xtab_multi_ifs_single_ips(file_reader_mmap_state_t* phan
 				*p = 0;
 
 				p++;
-				if (allow_repeat_ips) {
+				if (pstate->allow_repeat_ips) {
 					while (*p == ips)
 						p++;
 				}
@@ -398,11 +439,18 @@ lrec_t* lrec_parse_mmap_xtab_multi_ifs_single_ips(file_reader_mmap_state_t* phan
 	}
 }
 
-lrec_t* lrec_parse_mmap_xtab_multi_ifs_multi_ips(file_reader_mmap_state_t* phandle, char* ifs, char* ips,
-	int ifslen, int ipslen, int allow_repeat_ips)
+static lrec_t* lrec_parse_mmap_xtab_multi_ifs_multi_ips(file_reader_mmap_state_t* phandle,
+	lrec_reader_mmap_xtab_state_t* pstate)
 {
-	while (phandle->sol < phandle->eof && streqn(phandle->sol, ifs, ifslen))
+	char* ips = pstate->ips;
+	int ipslen = pstate->ipslen;
+	char* ifs = pstate->ifs;
+	int ifslen = pstate->ifslen;
+
+	// Skip blank lines
+	while (phandle->eof - phandle->sol >= ifslen && streqn(phandle->sol, ifs, ifslen)) {
 		phandle->sol += ifslen;
+	}
 
 	if (phandle->sol >= phandle->eof)
 		return NULL;
@@ -411,6 +459,9 @@ lrec_t* lrec_parse_mmap_xtab_multi_ifs_multi_ips(file_reader_mmap_state_t* phand
 
 	// Loop over fields, one per line
 	while (TRUE) {
+		if (phandle->sol >= phandle->eof)
+			break;
+
 		char* line  = phandle->sol;
 		char* key   = line;
 		char* value = "";
@@ -429,7 +480,7 @@ lrec_t* lrec_parse_mmap_xtab_multi_ifs_multi_ips(file_reader_mmap_state_t* phand
 				*p = 0;
 
 				p += ipslen;
-				if (allow_repeat_ips) {
+				if (pstate->allow_repeat_ips) {
 					while (streqn(p, ips, ipslen))
 						p += ipslen;
 				}
