@@ -10,6 +10,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include "cli/comment_handling.h"
 #include "lib/mlr_globals.h"
 #include "lib/mlrutil.h"
 #include "lib/string_builder.h"
@@ -45,6 +46,12 @@
 #define DQUOTE_IFS_STRIDX    0x2007
 #define DQUOTE_EOF_STRIDX    0x2008
 #define DQUOTE_DQUOTE_STRIDX 0x2009
+#define UTF8_BOM_STRIDX      0x200b
+
+#define UTF8_BOM "\xef\xbb\xbf"
+#define UTF8_BOM_LENGTH 3
+
+//#define DEBUG_PARSER
 
 // ----------------------------------------------------------------
 typedef struct _lrec_reader_stdio_csv_state_t {
@@ -57,6 +64,10 @@ typedef struct _lrec_reader_stdio_csv_state_t {
 	char* ifs_eof;
 	char* ifs;
 	int   do_auto_line_term;
+	comment_handling_t comment_handling;
+	char* comment_string;
+	int   comment_string_length;
+
 	char* dquote;
 	char* dquote_irs;
 	char* dquote_irs2;
@@ -70,6 +81,7 @@ typedef struct _lrec_reader_stdio_csv_state_t {
 	byte_reader_t*      pbr;
 	peek_file_reader_t* pfr;
 
+	parse_trie_t*       putf8_bom_parse_trie;
 	parse_trie_t*       pno_dquote_parse_trie;
 	parse_trie_t*       pdquote_parse_trie;
 
@@ -84,7 +96,7 @@ static void    lrec_reader_stdio_csv_free(lrec_reader_t* preader);
 static void    lrec_reader_stdio_csv_sof(void* pvstate, void* pvhandle);
 static lrec_t* lrec_reader_stdio_csv_process(void* pvstate, void* pvhandle, context_t* pctx);
 static int     lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pfields,
-	context_t* pctx);
+	context_t* pctx, int is_header);
 static lrec_t* paste_indices_and_data(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pdata_fields,
 	context_t* pctx);
 static lrec_t* paste_header_and_data(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pdata_fields,
@@ -93,7 +105,9 @@ static void*   lrec_reader_stdio_csv_open(void* pvstate, char* prepipe, char* fi
 static void    lrec_reader_stdio_csv_close(void* pvstate, void* pvhandle, char* prepipe);
 
 // ----------------------------------------------------------------
-lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implicit_header) {
+lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implicit_header,
+	comment_handling_t comment_handling, char* comment_string)
+{
 	lrec_reader_t* plrec_reader = mlr_malloc_or_die(sizeof(lrec_reader_t));
 
 	lrec_reader_stdio_csv_state_t* pstate = mlr_malloc_or_die(sizeof(lrec_reader_stdio_csv_state_t));
@@ -105,6 +119,10 @@ lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implici
 		pstate->do_auto_line_term = TRUE;
 	}
 
+	pstate->comment_handling = comment_handling;
+	pstate->comment_string   = comment_string;
+	pstate->comment_string_length = comment_string == NULL ? 0 : strlen(comment_string);
+
 	pstate->eof           = "\xff";
 	pstate->irs           = irs;
 	pstate->ifs           = ifs;
@@ -117,6 +135,12 @@ lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implici
 
 	pstate->dquotelen     = strlen(pstate->dquote);
 
+
+	// Parse trie for UTF-8 BOM
+	pstate->putf8_bom_parse_trie = parse_trie_alloc();
+	parse_trie_add_string(pstate->putf8_bom_parse_trie, UTF8_BOM, UTF8_BOM_STRIDX);
+
+	// Parse trie for non-double-quoted fields
 	pstate->pno_dquote_parse_trie = parse_trie_alloc();
 	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->eof,     EOF_STRIDX);
 	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->irs,     IRS_STRIDX);
@@ -124,6 +148,7 @@ lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implici
 	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->ifs,     IFS_STRIDX);
 	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->dquote,  DQUOTE_STRIDX);
 
+	// Parse trie for double-quoted fields
 	pstate->pdquote_parse_trie = parse_trie_alloc();
 	if (pstate->do_auto_line_term) {
 		pstate->dquote_irs  = mlr_paste_2_strings("\"", "\n");
@@ -141,10 +166,13 @@ lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implici
 	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_eof,    DQUOTE_EOF_STRIDX);
 	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_dquote, DQUOTE_DQUOTE_STRIDX);
 
+
 	pstate->pfields = rslls_alloc();
 	pstate->psb = sb_alloc(STRING_BUILDER_INIT_SIZE);
 	pstate->pbr = stdio_byte_reader_alloc();
-	pstate->pfr = pfr_alloc(pstate->pbr, mlr_imax2(pstate->pno_dquote_parse_trie->maxlen,
+	pstate->pfr = pfr_alloc(pstate->pbr, mlr_imax3(
+		pstate->putf8_bom_parse_trie->maxlen,
+		pstate->pno_dquote_parse_trie->maxlen,
 		pstate->pdquote_parse_trie->maxlen));
 
 	pstate->expect_header_line_next   = use_implicit_header ? FALSE : TRUE;
@@ -171,6 +199,7 @@ static void lrec_reader_stdio_csv_free(lrec_reader_t* preader) {
 	}
 	lhmslv_free(pstate->pheader_keepers);
 	pfr_free(pstate->pfr);
+	parse_trie_free(pstate->putf8_bom_parse_trie);
 	parse_trie_free(pstate->pno_dquote_parse_trie);
 	parse_trie_free(pstate->pdquote_parse_trie);
 	rslls_free(pstate->pfields);
@@ -195,49 +224,101 @@ static void lrec_reader_stdio_csv_sof(void* pvstate, void* pvhandle) {
 static lrec_t* lrec_reader_stdio_csv_process(void* pvstate, void* pvhandle, context_t* pctx) {
 	lrec_reader_stdio_csv_state_t* pstate = pvstate;
 
+	// Ingest the next header line, if expected
 	if (pstate->expect_header_line_next) {
-		if (!lrec_reader_stdio_csv_get_fields(pstate, pstate->pfields, pctx))
-			return NULL;
-		pstate->ilno++;
+		while (TRUE) {
+			if (!lrec_reader_stdio_csv_get_fields(pstate, pstate->pfields, pctx, TRUE))
+				return NULL;
+			pstate->ilno++;
 
-		slls_t* pheader_fields = slls_alloc();
-		int i = 0;
-		for (rsllse_t* pe = pstate->pfields->phead; i < pstate->pfields->length && pe != NULL; pe = pe->pnext) {
-			if (*pe->value == 0) {
-				fprintf(stderr, "%s: unacceptable empty CSV key at file \"%s\" line %lld.\n",
-					MLR_GLOBALS.bargv0, pctx->filename, pstate->ilno);
-				exit(1);
+			// We check for comments here rather than within the parser since it's important
+			// for users to be able to comment out lines containing double-quoted newlines.
+			if (pstate->comment_string != NULL && pstate->pfields->phead != NULL) {
+				if (streqn(pstate->pfields->phead->value, pstate->comment_string, pstate->comment_string_length)) {
+					if (pstate->comment_handling == PASS_COMMENTS) {
+						int i = 0;
+						for (
+							rsllse_t* pe = pstate->pfields->phead;
+							i < pstate->pfields->length && pe != NULL;
+							pe = pe->pnext, i++)
+						{
+							if (i > 0)
+								fputs(pstate->ifs, stdout);
+							fputs(pe->value, stdout);
+						}
+						if (pstate->do_auto_line_term) {
+							fputs(pctx->auto_line_term, stdout);
+						} else {
+							fputs(pstate->irs, stdout);
+						}
+					}
+					rslls_reset(pstate->pfields);
+					continue;
+				}
 			}
-			// Transfer pointer-free responsibility from the rslls to the
-			// header fields in the header keeper
-			if (string_starts_with(pe->value, "\xef\xbb\xbf")) {
-				// Strip UTF-8 BOM if any
-				slls_append(pheader_fields, mlr_strdup_or_die(&pe->value[3]), FREE_ENTRY_VALUE);
-				if (pe->free_flag & FREE_ENTRY_VALUE)
-					free(pe->value);
-			} else {
+
+			slls_t* pheader_fields = slls_alloc();
+			int i = 0;
+			for (rsllse_t* pe = pstate->pfields->phead; i < pstate->pfields->length && pe != NULL; pe = pe->pnext) {
+				if (*pe->value == 0) {
+					fprintf(stderr, "%s: unacceptable empty CSV key at file \"%s\" line %lld.\n",
+						MLR_GLOBALS.bargv0, pctx->filename, pstate->ilno);
+					exit(1);
+				}
+				// Transfer pointer-free responsibility from the rslls to the
+				// header fields in the header keeper
 				slls_append(pheader_fields, pe->value, pe->free_flag);
+				pe->free_flag = 0;
 			}
-			pe->free_flag = 0;
-		}
-		rslls_reset(pstate->pfields);
+			rslls_reset(pstate->pfields);
 
-		pstate->pheader_keeper = lhmslv_get(pstate->pheader_keepers, pheader_fields);
-		if (pstate->pheader_keeper == NULL) {
-			pstate->pheader_keeper = header_keeper_alloc(NULL, pheader_fields);
-			lhmslv_put(pstate->pheader_keepers, pheader_fields, pstate->pheader_keeper,
-				NO_FREE); // freed by header-keeper
-		} else { // Re-use the header-keeper in the header cache
-			slls_free(pheader_fields);
-		}
+			pstate->pheader_keeper = lhmslv_get(pstate->pheader_keepers, pheader_fields);
+			if (pstate->pheader_keeper == NULL) {
+				pstate->pheader_keeper = header_keeper_alloc(NULL, pheader_fields);
+				lhmslv_put(pstate->pheader_keepers, pheader_fields, pstate->pheader_keeper,
+					NO_FREE); // freed by header-keeper
+			} else { // Re-use the header-keeper in the header cache
+				slls_free(pheader_fields);
+			}
 
-		pstate->expect_header_line_next = FALSE;
+			pstate->expect_header_line_next = FALSE;
+			break;
+		}
 	}
-	int rc = lrec_reader_stdio_csv_get_fields(pstate, pstate->pfields, pctx);
-	pstate->ilno++;
-	if (rc == FALSE) // EOF
-		return NULL;
-	else {
+
+	// Ingest the next data line, if expected
+	while (TRUE) {
+		int rc = lrec_reader_stdio_csv_get_fields(pstate, pstate->pfields, pctx, FALSE);
+		pstate->ilno++;
+		if (rc == FALSE) // EOF
+			return NULL;
+
+		// We check for comments here rather than within the parser since it's important
+		// for users to be able to comment out lines containing double-quoted newlines.
+		if (pstate->comment_string != NULL && pstate->pfields->phead != NULL) {
+			if (streqn(pstate->pfields->phead->value, pstate->comment_string, pstate->comment_string_length)) {
+				if (pstate->comment_handling == PASS_COMMENTS) {
+					int i = 0;
+					for (
+						rsllse_t* pe = pstate->pfields->phead;
+						i < pstate->pfields->length && pe != NULL;
+						pe = pe->pnext, i++)
+					{
+						if (i > 0)
+							fputs(pstate->ifs, stdout);
+						fputs(pe->value, stdout);
+					}
+					if (pstate->do_auto_line_term) {
+						fputs(pctx->auto_line_term, stdout);
+					} else {
+						fputs(pstate->irs, stdout);
+					}
+				}
+				rslls_reset(pstate->pfields);
+				continue;
+			}
+		}
+
 		lrec_t* prec =  pstate->use_implicit_header
 			? paste_indices_and_data(pstate, pstate->pfields, pctx)
 			: paste_header_and_data(pstate, pstate->pfields, pctx);
@@ -247,7 +328,7 @@ static lrec_t* lrec_reader_stdio_csv_process(void* pvstate, void* pvhandle, cont
 }
 
 static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pfields,
-	context_t* pctx)
+	context_t* pctx, int is_header)
 {
 	int rc, stridx, matchlen, record_done, field_done;
 	peek_file_reader_t* pfr = pstate->pfr;
@@ -258,11 +339,29 @@ static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstat
 	if (pfr_peek_char(pfr) == (char)EOF) // char defaults to unsigned on some platforms
 		return FALSE;
 
-	// loop over fields in record
+	// Strip the UTF-8 BOM, if any. This is MUCH simpler for mmap, and for stdio on files.  For mmap
+	// we can test the first 3 bytes, then skip past them or not. For stdio on files we can fread
+	// the first 3 bytes, then rewind the fp if they're not the UTF-8 BOM. But for stdio on stdin
+	// (which is the primary reason we support stdio in Miller), we cannot rewind: stdin is not
+	// rewindable.
+	if (is_header) {
+		pfr_buffer_by(pfr, UTF8_BOM_LENGTH);
+		int rc = parse_trie_ring_match(pstate->putf8_bom_parse_trie,
+			pfr->peekbuf, pfr->sob, pfr->npeeked, pfr->peekbuflenmask,
+			&stridx, &matchlen);
+#ifdef DEBUG_PARSER
+		printf("RC=%d stridx=0x%04x matchlen=%d\n", rc, stridx, matchlen);
+#endif
+		if (rc == TRUE && stridx == UTF8_BOM_STRIDX) {
+			pfr_advance_by(pfr, matchlen);
+		}
+	}
+
+	// Loop over fields in record
 	record_done = FALSE;
 	while (!record_done) {
 		// Assumption is dquote is "\""
-		if (pfr_peek_char(pfr) != pstate->dquote[0]) {
+		if (pfr_peek_char(pfr) != pstate->dquote[0]) { // NOT DOUBLE-QUOTED
 
 			// Loop over characters in field
 			field_done = FALSE;
@@ -334,7 +433,7 @@ static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstat
 				}
 			}
 
-		} else {
+		} else { // DOUBLE-QUOTED
 			pfr_advance_by(pfr, pstate->dquotelen);
 
 			// loop over characters in field
