@@ -131,14 +131,14 @@ char* mlr_alloc_string_from_double(double value, char* fmt) {
 	return string;
 }
 
-char* mlr_alloc_string_from_ull(unsigned long  long value) {
+char* mlr_alloc_string_from_ull(unsigned long long value) {
 	int n = snprintf(NULL, 0, "%llu", value);
 	char* string = mlr_malloc_or_die(n+1);
 	sprintf(string, "%llu", value);
 	return string;
 }
 
-char* mlr_alloc_string_from_ll(long  long value) {
+char* mlr_alloc_string_from_ll(long long value) {
 	int n = snprintf(NULL, 0, "%lld", value);
 	char* string = mlr_malloc_or_die(n+1);
 	sprintf(string, "%lld", value);
@@ -166,7 +166,7 @@ char* mlr_alloc_string_from_char_range(char* start, int num_bytes) {
 	return string;
 }
 
-char* mlr_alloc_hexfmt_from_ll(long  long value) {
+char* mlr_alloc_hexfmt_from_ll(long long value) {
 	int n = snprintf(NULL, 0, "0x%llx", (unsigned long long)value);
 	char* string = mlr_malloc_or_die(n+1);
 	sprintf(string, "0x%llx", value);
@@ -362,6 +362,11 @@ int mlr_imax2(int a, int b) {
 		return b;
 }
 
+// ----------------------------------------------------------------
+int mlr_imax3(int a, int b, int c) {
+	return mlr_imax2(a, mlr_imax2(b, c));
+}
+
 // ----------------------------------------------------------------
 int power_of_two_above(int n) {
 	n |= (n >> 1);
@@ -482,13 +487,17 @@ char* mlr_alloc_double_backslash(char* input) {
 	for (p = input; *p; p++) {
 		input_length++;
 		if (*p == '\\') {
-			num_backslashes++;
+			if (p[1] != '.') {
+				num_backslashes++;
+			}
 		}
 	}
 	char* output = mlr_malloc_or_die(input_length + num_backslashes + 1);
 	for (p = input, q = output; *p; p++) {
 		if (*p == '\\') {
-			*(q++) = *p;
+			if (p[1] != '.') {
+				*(q++) = *p;
+			}
 			*(q++) = *p;
 		} else {
 			*(q++) = *p;
@@ -499,6 +508,17 @@ char* mlr_alloc_double_backslash(char* input) {
 	return output;
 }
 
+// ----------------------------------------------------------------
+// Returns -1 on error
+ssize_t get_file_size(char* filename) {
+	struct stat statbuf;
+	if (stat(filename, &statbuf) < 0) {
+		return (ssize_t)(-1);
+	} else {
+		return statbuf.st_size;
+	}
+}
+
 // ----------------------------------------------------------------
 char* read_file_into_memory(char* filename, size_t* psize) {
 	struct stat statbuf;
@@ -517,9 +537,9 @@ char* read_file_into_memory(char* filename, size_t* psize) {
 		return NULL;
 	}
 
-	int rc = fread(buffer, statbuf.st_size, 1, fp);
-	if (rc != 1) {
-		fprintf(stderr, "Unable t read content of %s\n", filename);
+	size_t rc = fread(buffer, 1, statbuf.st_size, fp);
+	if (rc != statbuf.st_size) {
+		fprintf(stderr, "Unable to read content of %s\n", filename);
 		perror("fread");
 		fprintf(stderr, "%s: could not fread \"%s\"\n", MLR_GLOBALS.bargv0, filename);
 		fclose(fp);
