@@ -1,4 +1,3 @@
-#include <regex.h>
 #include "lib/mlr_globals.h"
 #include "lib/mlrutil.h"
 #include "lib/mlrregex.h"
@@ -29,7 +28,7 @@ static mapper_t* mapper_having_fields_parse_cli(int* pargi, int argc, char** arg
 	cli_reader_opts_t* _, cli_writer_opts_t* __);
 
 static mapper_t* mapper_having_fields_alloc(slls_t* pfield_names, char* regex_string, criterion_t criterion);
-static void      mapper_having_fields_free(mapper_t* pmapper);
+static void      mapper_having_fields_free(mapper_t* pmapper, context_t* _);
 
 static sllv_t*   mapper_having_fields_at_least_process(lrec_t* pinrec, context_t* pctx, void* pvstate);
 static sllv_t*   mapper_having_fields_which_are_process(lrec_t* pinrec, context_t* pctx, void* pvstate);
@@ -44,6 +43,7 @@ mapper_setup_t mapper_having_fields_setup = {
 	.verb = "having-fields",
 	.pusage_func = mapper_having_fields_usage,
 	.pparse_func = mapper_having_fields_parse_cli,
+	.ignores_input = FALSE,
 };
 
 // ----------------------------------------------------------------
@@ -185,7 +185,7 @@ static mapper_t* mapper_having_fields_alloc(slls_t* pfield_names, char* regex_st
 	return pmapper;
 }
 
-static void mapper_having_fields_free(mapper_t* pmapper) {
+static void mapper_having_fields_free(mapper_t* pmapper, context_t* _) {
 	mapper_having_fields_state_t* pstate = pmapper->pvstate;
 	if (pstate->pfield_names != NULL)
 		slls_free(pstate->pfield_names);
