@@ -36,7 +36,7 @@ static mapper_t* mapper_nest_parse_cli(int* pargi, int argc, char** argv,
 static mapper_t* mapper_nest_alloc(ap_state_t* pargp, char* argv0,
 	char* field_name, char* nested_fs, char* nested_ps,
 	int do_explode, int do_pairs, int do_across_fields);
-static void    mapper_nest_free(mapper_t* pmapper);
+static void    mapper_nest_free(mapper_t* pmapper, context_t* _);
 
 static sllv_t* mapper_nest_explode_values_across_fields  (lrec_t* pinrec, context_t* pctx, void* pvstate);
 static sllv_t* mapper_nest_implode_values_across_fields  (lrec_t* pinrec, context_t* pctx, void* pvstate);
@@ -52,7 +52,8 @@ static void nest_bucket_free(nest_bucket_t* pbucket);
 mapper_setup_t mapper_nest_setup = {
 	.verb = "nest",
 	.pusage_func = mapper_nest_usage,
-	.pparse_func = mapper_nest_parse_cli
+	.pparse_func = mapper_nest_parse_cli,
+	.ignores_input = FALSE,
 };
 
 // ----------------------------------------------------------------
@@ -66,6 +67,7 @@ static void mapper_nest_usage(FILE* o, char* argv0, char* verb) {
 	fprintf(o, "  -f {field name}       Required.\n");
 	fprintf(o, "  --nested-fs {string}  Defaults to \";\". Field separator for nested values.\n");
 	fprintf(o, "  --nested-ps {string}  Defaults to \":\". Pair separator for nested key-value pairs.\n");
+	fprintf(o, "  --evar {string}       Shorthand for --explode --values ---across-records --nested-fs {string}\n");
 	fprintf(o, "Please use \"%s --usage-separator-options\" for information on specifying separators.\n",
 		argv0);
 
@@ -117,6 +119,7 @@ static mapper_t* mapper_nest_parse_cli(int* pargi, int argc, char** argv,
 	char* field_name = NULL;
 	char* nested_fs = ";";
 	char* nested_ps = ":";
+	char* vfs = NULL;
 	int   do_explode       = NEITHER_TRUE_NOR_FALSE;
 	int   do_pairs         = NEITHER_TRUE_NOR_FALSE;
 	int   do_across_fields = NEITHER_TRUE_NOR_FALSE;
@@ -133,12 +136,20 @@ static mapper_t* mapper_nest_parse_cli(int* pargi, int argc, char** argv,
 	ap_define_false_flag(pstate,  "--values",         &do_pairs);
 	ap_define_true_flag(pstate,   "--across-fields",  &do_across_fields);
 	ap_define_false_flag(pstate,  "--across-records", &do_across_fields);
+	ap_define_string_flag(pstate, "--evar",           &vfs);
 
 	if (!ap_parse(pstate, verb, pargi, argc, argv)) {
 		mapper_nest_usage(stderr, argv[0], verb);
 		return NULL;
 	}
 
+	if (vfs != NULL) {
+		do_explode = TRUE;
+		do_pairs = FALSE;
+		do_across_fields = FALSE;
+		nested_fs = vfs;
+	}
+
 	if (field_name == NULL) {
 		mapper_nest_usage(stderr, argv[0], verb);
 		return NULL;
@@ -191,9 +202,7 @@ static mapper_t* mapper_nest_alloc(ap_state_t* pargp, char* argv0,
 	} else {
 		if (do_pairs) {
 			// Should have been caught in CLI-parser.
-			fprintf(stderr, "%s: internal coding error detected at file %s line %d.\n",
-				argv0, __FILE__, __LINE__);
-			exit(1);
+			MLR_INTERNAL_CODING_ERROR();
 		} else {
 			pmapper->pprocess_func = do_across_fields
 				? mapper_nest_implode_values_across_fields
@@ -213,7 +222,7 @@ static mapper_t* mapper_nest_alloc(ap_state_t* pargp, char* argv0,
 	return pmapper;
 }
 
-static void mapper_nest_free(mapper_t* pmapper) {
+static void mapper_nest_free(mapper_t* pmapper, context_t* _) {
 	mapper_nest_state_t* pstate = pmapper->pvstate;
 
 	if (pstate->other_keys_to_other_values_to_buckets != NULL) {
@@ -251,14 +260,18 @@ static sllv_t* mapper_nest_explode_values_across_fields(lrec_t* pinrec, context_
 	lrece_t* porig = pentry;
 
 	char* sep = pstate->nested_fs;
+	int seplen = strlen(sep);
 	int i = 1;
-	for (char* piece = strtok(field_value, sep); piece != NULL; piece = strtok(NULL, sep), i++) {
+	char* walker = field_value;
+	char* piece = NULL;
+	while ((piece = mlr_strmsep(&walker, sep, seplen)) != NULL) {
 		char  istring_free_flags;
-		char* istring = make_nidx_key(i, &istring_free_flags);
+		char* istring = low_int_to_string(i, &istring_free_flags);
 		char* new_key = mlr_paste_3_strings(pstate->field_name, "_", istring);
 		if (istring_free_flags & FREE_ENTRY_KEY)
 			free(istring);
 		pentry = lrec_put_after(pinrec, pentry, new_key, mlr_strdup_or_die(piece), FREE_ENTRY_KEY|FREE_ENTRY_VALUE);
+		i++;
 	}
 	lrec_unlink_and_free(pinrec, porig);
 	return sllv_single(pinrec);;
@@ -315,8 +328,10 @@ static sllv_t* mapper_nest_explode_values_across_records(lrec_t* pinrec, context
 
 	sllv_t* poutrecs = sllv_alloc();
 	char* sep = pstate->nested_fs;
-	int i = 1;
-	for (char* piece = strtok(field_value, sep); piece != NULL; piece = strtok(NULL, sep), i++) {
+	int seplen = strlen(sep);
+	char* walker = field_value;
+	char* piece = NULL;
+	while ((piece = mlr_strmsep(&walker, sep, seplen)) != NULL) {
 		lrec_t* poutrec = lrec_copy(pinrec);
 		lrec_put(poutrec, pstate->field_name, mlr_strdup_or_die(piece), FREE_ENTRY_VALUE);
 		sllv_append(poutrecs, poutrec);
@@ -402,7 +417,10 @@ static sllv_t* mapper_nest_explode_pairs_across_fields(lrec_t* pinrec, context_t
 	lrece_t* porig = pentry;
 
 	char* sep = pstate->nested_fs;
-	for (char* piece = strtok(field_value, sep); piece != NULL; piece = strtok(NULL, sep)) {
+	int seplen = strlen(sep);
+	char* walker = field_value;
+	char* piece = NULL;
+	while ((piece = mlr_strmsep(&walker, sep, seplen)) != NULL) {
 		char* found_sep = strstr(piece, pstate->nested_ps);
 		if (found_sep != NULL) { // there is a pair
 			*found_sep = 0;
@@ -432,7 +450,10 @@ static sllv_t* mapper_nest_explode_pairs_across_records(lrec_t* pinrec, context_
 
 	sllv_t* poutrecs = sllv_alloc();
 	char* sep = pstate->nested_fs;
-	for (char* piece = strtok(field_value, sep); piece != NULL; piece = strtok(NULL, sep)) {
+	int seplen = strlen(sep);
+	char* walker = field_value;
+	char* piece = NULL;
+	while ((piece = mlr_strmsep(&walker, sep, seplen)) != NULL) {
 		char* found_sep = strstr(piece, pstate->nested_ps);
 		lrec_t* poutrec = lrec_copy(pinrec);
 		lrece_t* pe = NULL;
