@@ -1,6 +1,6 @@
 #include "lib/mlrutil.h"
 #include "containers/lrec.h"
-#include "containers/mvfuncs.h"
+#include "lib/mvfuncs.h"
 #include "containers/slls.h"
 #include "containers/mixutil.h"
 #include "cli/argparse.h"
