@@ -60,7 +60,7 @@ static mapper_t* mapper_stats2_parse_cli(int* pargi, int argc, char** argv,
 static mapper_t* mapper_stats2_alloc(ap_state_t* pargp, slls_t* paccumulator_names,
 	string_array_t* pvalue_field_name_pairs, slls_t* pgroup_by_field_names,
 	int do_verbose, int do_iterative_stats, int do_hold_and_fit);
-static void      mapper_stats2_free(mapper_t* pmapper);
+static void      mapper_stats2_free(mapper_t* pmapper, context_t* _);
 static sllv_t*   mapper_stats2_process(lrec_t* pinrec, context_t* pctx, void* pvstate);
 static void      mapper_stats2_ingest(lrec_t* pinrec, context_t* pctx, mapper_stats2_state_t* pstate);
 static sllv_t*   mapper_stats2_emit_all(mapper_stats2_state_t* pstate);
@@ -99,7 +99,8 @@ static int stats2_acc_lookup_table_length = sizeof(stats2_acc_lookup_table) / si
 mapper_setup_t mapper_stats2_setup = {
 	.verb = "stats2",
 	.pusage_func = mapper_stats2_usage,
-	.pparse_func = mapper_stats2_parse_cli
+	.pparse_func = mapper_stats2_parse_cli,
+	.ignores_input = FALSE,
 };
 
 // ----------------------------------------------------------------
@@ -200,7 +201,7 @@ static mapper_t* mapper_stats2_alloc(ap_state_t* pargp, slls_t* paccumulator_nam
 	return pmapper;
 }
 
-static void mapper_stats2_free(mapper_t* pmapper) {
+static void mapper_stats2_free(mapper_t* pmapper, context_t* _) {
 	mapper_stats2_state_t* pstate = pmapper->pvstate;
 	slls_free(pstate->paccumulator_names);
 	string_array_free(pstate->pvalue_field_name_pairs);
