@@ -8,8 +8,8 @@ lrec_writer_t*  lrec_writer_alloc(cli_writer_opts_t* popts) {
 
 	} else if (streq(popts->ofile_fmt, "json")) {
 		return lrec_writer_json_alloc(popts->stack_json_output_vertically,
-			popts->wrap_json_output_in_outer_list, popts->quote_json_values_always,
-			popts->output_json_flatten_separator);
+			popts->wrap_json_output_in_outer_list, popts->json_quote_int_keys,
+			popts->json_quote_non_string_values, popts->output_json_flatten_separator, popts->ors);
 
 	} else if (streq(popts->ofile_fmt, "csv")) {
 		return lrec_writer_csv_alloc(popts->ors, popts->ofs, popts->oquoting,
@@ -33,7 +33,8 @@ lrec_writer_t*  lrec_writer_alloc(cli_writer_opts_t* popts) {
 				MLR_GLOBALS.bargv0, popts->ofs);
 			return NULL;
 		} else {
-			return lrec_writer_pprint_alloc(popts->ors, popts->ofs[0], popts->right_align_pprint);
+			return lrec_writer_pprint_alloc(popts->ors, popts->ofs[0], popts->right_align_pprint,
+				popts->pprint_barred);
 		}
 
 	} else {
@@ -44,18 +45,14 @@ lrec_writer_t*  lrec_writer_alloc(cli_writer_opts_t* popts) {
 // ----------------------------------------------------------------
 lrec_writer_t* lrec_writer_alloc_or_die(cli_writer_opts_t* popts) {
 	lrec_writer_t* plrec_writer = lrec_writer_alloc(popts);
-	if (plrec_writer == NULL) {
-		fprintf(stderr, "%s: internal coding error detected in file \"%s\" at line %d.\n",
-			MLR_GLOBALS.bargv0, __FILE__, __LINE__);
-		exit(1);
-	}
+	MLR_INTERNAL_CODING_ERROR_IF(plrec_writer == NULL);
 	return plrec_writer;
 }
 
 // ----------------------------------------------------------------
-void lrec_writer_print_all(lrec_writer_t* pwriter, FILE* fp, sllv_t* poutrecs) {
+void lrec_writer_print_all(lrec_writer_t* pwriter, FILE* fp, sllv_t* poutrecs, context_t* pctx) {
 	while (poutrecs->phead != NULL) {
 		lrec_t* poutrec = sllv_pop(poutrecs);
-		pwriter->pprocess_func(pwriter->pvstate, fp, poutrec);
+		pwriter->pprocess_func(pwriter->pvstate, fp, poutrec, pctx);
 	}
 }
