@@ -1,6 +1,7 @@
 #include <stdio.h>
 #include <string.h>
 #include "lib/minunit.h"
+#include "lib/mlr_globals.h"
 #include "lib/mlrutil.h"
 #include "containers/lrec.h"
 #include "containers/sllv.h"
@@ -552,6 +553,7 @@ static char * run_all_tests() {
 }
 
 int main(int argc, char **argv) {
+	mlr_global_init(argv[0], NULL);
 	printf("TEST_JOIN_BUCKET_KEEPER ENTER\n");
 	for (int argi = 1; argi < argc; argi++) {
 		if (streq(argv[argi], "-v"))
