@@ -1,6 +1,7 @@
 #include <stdio.h>
 #include <string.h>
 #include "lib/minunit.h"
+#include "lib/mlr_globals.h"
 #include "lib/mlrutil.h"
 #include "containers/lrec.h"
 #include "containers/sllv.h"
@@ -70,9 +71,8 @@ static char* test_lrec_unbacked_api() {
 // ----------------------------------------------------------------
 static char* test_lrec_dkvp_api() {
 	char* line = mlr_strdup_or_die("w=2,x=3,y=4,z=5");
-	context_t ctx = { .nr = 777, .fnr = 888, .filenum = 999, .filename = "test-file" };
 
-	lrec_t* prec = lrec_parse_stdio_dkvp_single_sep(line, ',', '=', FALSE, &ctx);
+	lrec_t* prec = lrec_parse_stdio_dkvp_single_sep(line, ',', '=', FALSE);
 	mu_assert_lf(prec->field_count == 4);
 
 	mu_assert_lf(streq(lrec_get(prec, "w"), "2"));
@@ -414,6 +414,7 @@ static char * run_all_tests() {
 }
 
 int main(int argc, char **argv) {
+	mlr_global_init(argv[0], NULL);
 	printf("TEST_LREC ENTER\n");
 	char *result = run_all_tests();
 	printf("\n");
