@@ -1,6 +1,7 @@
 #include "lib/mlr_globals.h"
 #include "lib/minunit.h"
-#include "mapping/rval_evaluators.h"
+#include "dsl/rval_evaluators.h"
+#include "dsl/function_manager.h"
 
 // ----------------------------------------------------------------
 int tests_run         = 0;
@@ -12,7 +13,9 @@ int assertions_failed = 0;
 static char * test_caps() {
 	printf("\n");
 	printf("-- TEST_RVAL_EVALUATORS test_caps ENTER\n");
-	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE};
+	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE,
+		.ips = "=", .ifs = ",", .irs = "\n", .ops = "=", .ofs = ",", .ors = "\n", .auto_line_term = "\n"
+	};
 	context_t* pctx = &ctx;
 
 	rval_evaluator_t* pnr       = rval_evaluator_alloc_from_NR();
@@ -22,9 +25,8 @@ static char * test_caps() {
 
 	lrec_t* prec = lrec_unbacked_alloc();
 	lhmsmv_t* ptyped_overlay = lhmsmv_alloc();
-	mlhmmv_t* poosvars = mlhmmv_alloc();
+	mlhmmv_root_t* poosvars = mlhmmv_root_alloc();
 	string_array_t* pregex_captures = NULL;
-	bind_stack_t* pbind_stack = bind_stack_alloc();
 	loop_stack_t* ploop_stack = loop_stack_alloc();
 
 	variables_t variables = (variables_t) {
@@ -33,7 +35,6 @@ static char * test_caps() {
 		.poosvars         = poosvars,
 		.ppregex_captures = &pregex_captures,
 		.pctx             = pctx,
-		.pbind_stack      = pbind_stack,
 		.ploop_stack      = ploop_stack,
 	};
 
@@ -65,20 +66,21 @@ static char * test_caps() {
 static char * test_strings() {
 	printf("\n");
 	printf("-- TEST_RVAL_EVALUATORS test_strings ENTER\n");
-	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE};
+	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE,
+		.ips = "=", .ifs = ",", .irs = "\n", .ops = "=", .ofs = ",", .ors = "\n", .auto_line_term = "\n"
+	};
 	context_t* pctx = &ctx;
 
 	rval_evaluator_t* ps       = rval_evaluator_alloc_from_field_name("s", TYPE_INFER_STRING_FLOAT_INT);
-	rval_evaluator_t* pdef     = rval_evaluator_alloc_from_strnum_literal("def", TYPE_INFER_STRING_FLOAT_INT);
+	rval_evaluator_t* pdef     = rval_evaluator_alloc_from_string_literal("def");
 	rval_evaluator_t* pdot     = rval_evaluator_alloc_from_x_ss_func(s_xx_dot_func, ps, pdef);
 	rval_evaluator_t* ptolower = rval_evaluator_alloc_from_s_s_func(s_s_tolower_func, pdot);
 	rval_evaluator_t* ptoupper = rval_evaluator_alloc_from_s_s_func(s_s_toupper_func, pdot);
 
 	lrec_t* prec = lrec_unbacked_alloc();
 	lhmsmv_t* ptyped_overlay = lhmsmv_alloc();
-	mlhmmv_t* poosvars = mlhmmv_alloc();
+	mlhmmv_root_t* poosvars = mlhmmv_root_alloc();
 	string_array_t* pregex_captures = NULL;
-	bind_stack_t* pbind_stack = bind_stack_alloc();
 	loop_stack_t* ploop_stack = loop_stack_alloc();
 
 	lrec_put(prec, "s", "abc", NO_FREE);
@@ -90,7 +92,6 @@ static char * test_strings() {
 		.poosvars         = poosvars,
 		.ppregex_captures = &pregex_captures,
 		.pctx             = pctx,
-		.pbind_stack      = pbind_stack,
 		.ploop_stack      = ploop_stack,
 	};
 
@@ -131,10 +132,12 @@ static char * test_strings() {
 static char * test_numbers() {
 	printf("\n");
 	printf("-- TEST_RVAL_EVALUATORS test_numbers ENTER\n");
-	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE};
+	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE,
+		.ips = "=", .ifs = ",", .irs = "\n", .ops = "=", .ofs = ",", .ors = "\n", .auto_line_term = "\n"
+	};
 	context_t* pctx = &ctx;
 
-	rval_evaluator_t* p2     = rval_evaluator_alloc_from_strnum_literal("2.0", TYPE_INFER_STRING_FLOAT_INT);
+	rval_evaluator_t* p2     = rval_evaluator_alloc_from_numeric_literal("2.0");
 	rval_evaluator_t* px     = rval_evaluator_alloc_from_field_name("x", TYPE_INFER_STRING_FLOAT_INT);
 	rval_evaluator_t* plogx  = rval_evaluator_alloc_from_f_f_func(f_f_log10_func, px);
 	rval_evaluator_t* p2logx = rval_evaluator_alloc_from_x_xx_func(x_xx_times_func, p2, plogx);
@@ -142,19 +145,21 @@ static char * test_numbers() {
 	rval_evaluator_t* p4     = rval_evaluator_alloc_from_x_xx_func(x_xx_times_func, p2, p2);
 
 	mlr_dsl_ast_node_t* pxnode     = mlr_dsl_ast_node_alloc("x",  MD_AST_NODE_TYPE_FIELD_NAME);
-	mlr_dsl_ast_node_t* plognode   = mlr_dsl_ast_node_alloc_zary("log", MD_AST_NODE_TYPE_NON_SIGIL_NAME);
+	mlr_dsl_ast_node_t* plognode   = mlr_dsl_ast_node_alloc_zary("log", MD_AST_NODE_TYPE_FUNCTION_CALLSITE);
 	mlr_dsl_ast_node_t* plogxnode  = mlr_dsl_ast_node_append_arg(plognode, pxnode);
-	mlr_dsl_ast_node_t* p2node     = mlr_dsl_ast_node_alloc("2",   MD_AST_NODE_TYPE_STRNUM_LITERAL);
+	mlr_dsl_ast_node_t* p2node     = mlr_dsl_ast_node_alloc("2",   MD_AST_NODE_TYPE_NUMERIC_LITERAL);
 	mlr_dsl_ast_node_t* p2logxnode = mlr_dsl_ast_node_alloc_binary("*", MD_AST_NODE_TYPE_OPERATOR,
 		p2node, plogxnode);
 
-	rval_evaluator_t*  pastr = rval_evaluator_alloc_from_ast(p2logxnode, TYPE_INFER_STRING_FLOAT_INT, 0);
+	fmgr_t* pfmgr = fmgr_alloc();
+	rval_evaluator_t*  pastr = rval_evaluator_alloc_from_ast(p2logxnode, pfmgr, TYPE_INFER_STRING_FLOAT_INT, 0);
+	fmgr_resolve_func_callsites(pfmgr);
+	fmgr_free(pfmgr, &ctx);
 
 	lrec_t* prec = lrec_unbacked_alloc();
 	lhmsmv_t* ptyped_overlay = lhmsmv_alloc();
-	mlhmmv_t* poosvars = mlhmmv_alloc();
+	mlhmmv_root_t* poosvars = mlhmmv_root_alloc();
 	string_array_t* pregex_captures = NULL;
-	bind_stack_t* pbind_stack = bind_stack_alloc();
 	loop_stack_t* ploop_stack = loop_stack_alloc();
 
 	lrec_put(prec, "x", "4.5", NO_FREE);
@@ -165,7 +170,6 @@ static char * test_numbers() {
 		.poosvars         = poosvars,
 		.ppregex_captures = &pregex_captures,
 		.pctx             = pctx,
-		.pbind_stack      = pbind_stack,
 		.ploop_stack      = ploop_stack,
 	};
 
@@ -184,7 +188,6 @@ static char * test_numbers() {
 	printf("newval log(x)   = %s\n", mv_describe_val(valplogx));
 	printf("newval 2*log(x) = %s\n", mv_describe_val(valp2logx));
 
-	printf("XXX %s\n", mt_describe_type(valp2.type));
 	mu_assert_lf(valp2.type     == MT_FLOAT);
 	mu_assert_lf(valp4.type     == MT_FLOAT);
 	mu_assert_lf(valpx.type     == MT_FLOAT);
@@ -238,14 +241,15 @@ static char * test_numbers() {
 static char * test_logical_and() {
 	printf("\n");
 	printf("-- TEST_RVAL_EVALUATORS test4 ENTER\n");
-	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE};
+	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE,
+		.ips = "=", .ifs = ",", .irs = "\n", .ops = "=", .ofs = ",", .ors = "\n", .auto_line_term = "\n"
+	};
 	context_t* pctx = &ctx;
 
 	lrec_t* prec = NULL;
 	lhmsmv_t* ptyped_overlay = NULL;
-	mlhmmv_t* poosvars = NULL;
+	mlhmmv_root_t* poosvars = NULL;
 	string_array_t* pregex_captures = NULL;
-	bind_stack_t* pbind_stack = bind_stack_alloc();
 	loop_stack_t* ploop_stack = loop_stack_alloc();
 
 	variables_t variables = (variables_t) {
@@ -254,7 +258,6 @@ static char * test_logical_and() {
 		.poosvars         = poosvars,
 		.ppregex_captures = &pregex_captures,
 		.pctx             = pctx,
-		.pbind_stack      = pbind_stack,
 		.ploop_stack      = ploop_stack,
 	};
 
@@ -286,14 +289,14 @@ static char * test_logical_and() {
 	mv_t ofa = pfa->pprocess_func(pfa->pvstate, &variables);
 	mv_t oaa = paa->pprocess_func(paa->pvstate, &variables);
 
-	mu_assert_lf(ott.type == MT_BOOL); mu_assert_lf(ott.u.boolv == TRUE);
-	mu_assert_lf(otf.type == MT_BOOL); mu_assert_lf(otf.u.boolv == FALSE);
-	mu_assert_lf(oft.type == MT_BOOL); mu_assert_lf(oft.u.boolv == FALSE);
-	mu_assert_lf(off.type == MT_BOOL); mu_assert_lf(off.u.boolv == FALSE);
-	mu_assert_lf(oat.type == MT_BOOL); mu_assert_lf(oat.u.boolv == TRUE);
-	mu_assert_lf(oaf.type == MT_BOOL); mu_assert_lf(oaf.u.boolv == FALSE);
-	mu_assert_lf(ota.type == MT_BOOL); mu_assert_lf(ota.u.boolv == TRUE);
-	mu_assert_lf(ofa.type == MT_BOOL); mu_assert_lf(ofa.u.boolv == FALSE);
+	mu_assert_lf(ott.type == MT_BOOLEAN); mu_assert_lf(ott.u.boolv == TRUE);
+	mu_assert_lf(otf.type == MT_BOOLEAN); mu_assert_lf(otf.u.boolv == FALSE);
+	mu_assert_lf(oft.type == MT_BOOLEAN); mu_assert_lf(oft.u.boolv == FALSE);
+	mu_assert_lf(off.type == MT_BOOLEAN); mu_assert_lf(off.u.boolv == FALSE);
+	mu_assert_lf(oat.type == MT_BOOLEAN); mu_assert_lf(oat.u.boolv == TRUE);
+	mu_assert_lf(oaf.type == MT_BOOLEAN); mu_assert_lf(oaf.u.boolv == FALSE);
+	mu_assert_lf(ota.type == MT_BOOLEAN); mu_assert_lf(ota.u.boolv == TRUE);
+	mu_assert_lf(ofa.type == MT_BOOLEAN); mu_assert_lf(ofa.u.boolv == FALSE);
 	mu_assert_lf(oaa.type == MT_ABSENT);
 
 	return 0;
@@ -303,14 +306,15 @@ static char * test_logical_and() {
 static char * test_logical_or() {
 	printf("\n");
 	printf("-- TEST_RVAL_EVALUATORS test4 ENTER\n");
-	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE};
+	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE,
+		.ips = "=", .ifs = ",", .irs = "\n", .ops = "=", .ofs = ",", .ors = "\n", .auto_line_term = "\n"
+	};
 	context_t* pctx = &ctx;
 
 	lrec_t* prec = NULL;
 	lhmsmv_t* ptyped_overlay = NULL;
-	mlhmmv_t* poosvars = NULL;
+	mlhmmv_root_t* poosvars = NULL;
 	string_array_t* pregex_captures = NULL;
-	bind_stack_t* pbind_stack = bind_stack_alloc();
 	loop_stack_t* ploop_stack = loop_stack_alloc();
 
 	variables_t variables = (variables_t) {
@@ -319,7 +323,6 @@ static char * test_logical_or() {
 		.poosvars         = poosvars,
 		.ppregex_captures = &pregex_captures,
 		.pctx             = pctx,
-		.pbind_stack      = pbind_stack,
 		.ploop_stack      = ploop_stack,
 	};
 
@@ -351,14 +354,14 @@ static char * test_logical_or() {
 	mv_t oaf = paf->pprocess_func(paf->pvstate, &variables);
 	mv_t oaa = paa->pprocess_func(paa->pvstate, &variables);
 
-	mu_assert_lf(ott.type == MT_BOOL); mu_assert_lf(ott.u.boolv == TRUE);
-	mu_assert_lf(otf.type == MT_BOOL); mu_assert_lf(otf.u.boolv == TRUE);
-	mu_assert_lf(ota.type == MT_BOOL); mu_assert_lf(ota.u.boolv == TRUE);
-	mu_assert_lf(oft.type == MT_BOOL); mu_assert_lf(oft.u.boolv == TRUE);
-	mu_assert_lf(off.type == MT_BOOL); mu_assert_lf(off.u.boolv == FALSE);
-	mu_assert_lf(ofa.type == MT_BOOL); mu_assert_lf(ofa.u.boolv == FALSE);
-	mu_assert_lf(oat.type == MT_BOOL); mu_assert_lf(oat.u.boolv == TRUE);
-	mu_assert_lf(oaf.type == MT_BOOL); mu_assert_lf(oaf.u.boolv == FALSE);
+	mu_assert_lf(ott.type == MT_BOOLEAN); mu_assert_lf(ott.u.boolv == TRUE);
+	mu_assert_lf(otf.type == MT_BOOLEAN); mu_assert_lf(otf.u.boolv == TRUE);
+	mu_assert_lf(ota.type == MT_BOOLEAN); mu_assert_lf(ota.u.boolv == TRUE);
+	mu_assert_lf(oft.type == MT_BOOLEAN); mu_assert_lf(oft.u.boolv == TRUE);
+	mu_assert_lf(off.type == MT_BOOLEAN); mu_assert_lf(off.u.boolv == FALSE);
+	mu_assert_lf(ofa.type == MT_BOOLEAN); mu_assert_lf(ofa.u.boolv == FALSE);
+	mu_assert_lf(oat.type == MT_BOOLEAN); mu_assert_lf(oat.u.boolv == TRUE);
+	mu_assert_lf(oaf.type == MT_BOOLEAN); mu_assert_lf(oaf.u.boolv == FALSE);
 	mu_assert_lf(oaa.type == MT_ABSENT);
 
 	return 0;
@@ -368,14 +371,15 @@ static char * test_logical_or() {
 static char * test_logical_xor() {
 	printf("\n");
 	printf("-- TEST_RVAL_EVALUATORS test4 ENTER\n");
-	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE};
+	context_t ctx = {.nr = 888, .fnr = 999, .filenum = 123, .filename = "filename-goes-here", .force_eof = FALSE,
+		.ips = "=", .ifs = ",", .irs = "\n", .ops = "=", .ofs = ",", .ors = "\n", .auto_line_term = "\n"
+	};
 	context_t* pctx = &ctx;
 
 	lrec_t* prec = NULL;
 	lhmsmv_t* ptyped_overlay = NULL;
-	mlhmmv_t* poosvars = NULL;
+	mlhmmv_root_t* poosvars = NULL;
 	string_array_t* pregex_captures = NULL;
-	bind_stack_t* pbind_stack = bind_stack_alloc();
 	loop_stack_t* ploop_stack = loop_stack_alloc();
 
 	variables_t variables = (variables_t) {
@@ -384,7 +388,6 @@ static char * test_logical_xor() {
 		.poosvars         = poosvars,
 		.ppregex_captures = &pregex_captures,
 		.pctx             = pctx,
-		.pbind_stack      = pbind_stack,
 		.ploop_stack      = ploop_stack,
 	};
 
@@ -416,14 +419,14 @@ static char * test_logical_xor() {
 	mv_t ofa = pfa->pprocess_func(pfa->pvstate, &variables);
 	mv_t oaa = paa->pprocess_func(paa->pvstate, &variables);
 
-	mu_assert_lf(ott.type == MT_BOOL); mu_assert_lf(ott.u.boolv == FALSE);
-	mu_assert_lf(otf.type == MT_BOOL); mu_assert_lf(otf.u.boolv == TRUE);
-	mu_assert_lf(oft.type == MT_BOOL); mu_assert_lf(oft.u.boolv == TRUE);
-	mu_assert_lf(off.type == MT_BOOL); mu_assert_lf(off.u.boolv == FALSE);
-	mu_assert_lf(oat.type == MT_BOOL); mu_assert_lf(oat.u.boolv == TRUE);
-	mu_assert_lf(oaf.type == MT_BOOL); mu_assert_lf(oaf.u.boolv == FALSE);
-	mu_assert_lf(ota.type == MT_BOOL); mu_assert_lf(ota.u.boolv == TRUE);
-	mu_assert_lf(ofa.type == MT_BOOL); mu_assert_lf(ofa.u.boolv == FALSE);
+	mu_assert_lf(ott.type == MT_BOOLEAN); mu_assert_lf(ott.u.boolv == FALSE);
+	mu_assert_lf(otf.type == MT_BOOLEAN); mu_assert_lf(otf.u.boolv == TRUE);
+	mu_assert_lf(oft.type == MT_BOOLEAN); mu_assert_lf(oft.u.boolv == TRUE);
+	mu_assert_lf(off.type == MT_BOOLEAN); mu_assert_lf(off.u.boolv == FALSE);
+	mu_assert_lf(oat.type == MT_BOOLEAN); mu_assert_lf(oat.u.boolv == TRUE);
+	mu_assert_lf(oaf.type == MT_BOOLEAN); mu_assert_lf(oaf.u.boolv == FALSE);
+	mu_assert_lf(ota.type == MT_BOOLEAN); mu_assert_lf(ota.u.boolv == TRUE);
+	mu_assert_lf(ofa.type == MT_BOOLEAN); mu_assert_lf(ofa.u.boolv == FALSE);
 	mu_assert_lf(oaa.type == MT_ABSENT);
 
 	return 0;
